#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from dataclasses import dataclass


@dataclass
class FileInfo:
    file_name: str
    last_modified: str
    file_size: int



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import TypeVar


FIREWALL_RULE_INITIAL_PORT = 5000
FIREWALL_RULE_FINAL_PORT = 15500


AvailabilityZone = TypeVar("AvailabilityZone", str, bytes)


CONTAINER_CPU = 4096
CONTAINER_MEMORY = 30720
CONTAINER_IMAGE = "539290649537.dkr.ecr.us-west-2.amazonaws.com/one-docker-prod:latest"
TASK_POLICY = {
    "Version": "2012-10-17",
    "Statement": [
        {
            "Action": "sts:AssumeRole",
            "Principal": {"Service": "ecs-tasks.amazonaws.com"},
            "Effect": "Allow",
            "Sid": "",
        }
    ],
}
TASK_EXECUTION_POLICY = {
    "Version": "2012-10-17",
    "Statement": [
        {
            "Action": "sts:AssumeRole",
            "Principal": {"Service": "ecs-tasks.amazonaws.com"},
            "Effect": "Allow",
            "Sid": "",
        }
    ],
}



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import abc
from datetime import date
from typing import Optional

from fbpcp.entity.cloud_cost import CloudCost


class BillingService(abc.ABC):
    @abc.abstractmethod
    def get_cost(
        self, start_date: date, end_date: date, region: Optional[str] = None
    ) -> CloudCost:
        pass



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

# patternlint-disable f-string-may-be-missing-leading-f

from enum import Enum

from pce.validator.pce_standard_constants import (
    FIREWALL_RULE_INITIAL_PORT,
    FIREWALL_RULE_FINAL_PORT,
    CONTAINER_CPU,
    CONTAINER_MEMORY,
    CONTAINER_IMAGE,
)


class ValidationErrorDescriptionTemplate(Enum):
    UNKNOWN = "Unknown error"
    VPC_PEERING_NO_VPC_PEERING = "No VPC peering set."
    VPC_PEERING_NO_VPC = "No VPC set."
    VPC_PEERING_NO_VPC_CIDR = "No VPC CIDR set."
    VPC_PEERING_NO_ROUTE_TABLE = "No route table set."
    # @lint-ignore This is a valid/inteneded usage of a string as a template
    FIREWALL_CIDR_NOT_OVERLAPS_VPC = "VPC peering for VPC {peer_target_id} doesn't have an inbound rule to allow traffic to {vpc_id}:{vpc_cidr}."
    FIREWALL_CIDR_CANT_CONTAIN_EXPECTED_RANGE = f"Ingress cidr {{fr_vpc_id}}:{{fri_cidr}}:{{fri_from_port}}-{{fri_to_port}} can't contain the expected port range {FIREWALL_RULE_INITIAL_PORT}-{FIREWALL_RULE_FINAL_PORT}"
    FIREWALL_INVALID_RULESETS = "Invalid firewall rulesets: {error_reasons}"
    ROUTE_TABLE_VPC_PEERING_MISSING = "No valid VPC peering found in route table."
    CLUSTER_DEFINITION_NOT_SET = "No container definition."
    CLUSTER_DEFINITION_WRONG_VALUES = (
        "Container values incorrectly set: {error_reasons}"
    )
    NON_PRIVATE_VPC_CIDR = "The CIDR of the vpc {vpc_cidr} is not a private range."
    NOT_ALL_AZ_USED = "Not all availability zones used."
    FIREWALL_PEER_ROUTE_NOT_SET = "No peer route found."
    FIREWALL_RULES_NOT_FOUND = "No firewall rules found tagged with {pce_id}."
    CLUSTER_DEFINITION_WRONG_VALUE = (
        "{resource_name} value '{value}' is incorrect, expected '{expected_value}'."
    )


class ValidationErrorSolutionHintTemplate(Enum):
    FIREWALL_INVALID_RULESETS = (
        "Set correct CIDR and port ranges for the offending firewall rules."
    )
    ROUTE_TABLE_VPC_PEERING_MISSING = "Define a VPC connection in the route table."
    CLUSTER_DEFINITION_WRONG_VALUES = f"Please set container values (cpu, memory, image) as ({CONTAINER_CPU},{CONTAINER_MEMORY},'{CONTAINER_IMAGE}')"
    NON_PRIVATE_VPC_CIDR = "Set a private CIDR (https://en.wikipedia.org/wiki/Private_network) for the vpc."
    NOT_ALL_AZ_USED = (
        "Set the subnets so that all availability zones from {region} ({azs}) are used."
    )


class ValidationWarningDescriptionTemplate(Enum):
    VPC_PEERING_PEERING_NOT_READY = "Still setting up peering."
    FIREWALL_CIDR_EXCEED_EXPECTED_RANGE = f"Ingress cidr {{fr_vpc_id}}:{{fri_cidr}}:{{fri_from_port}}-{{fri_to_port}} exceeds the expected port range {FIREWALL_RULE_INITIAL_PORT}-{FIREWALL_RULE_FINAL_PORT}"
    FIREWALL_FLAGGED_RULESETS = (
        "These issues are not fatal but are worth noticing: {warning_reasons}"
    )
    CLUSTER_DEFINITION_FLAGGED_VALUE = (
        "{resource_name} value '{value}' is not expected, should be '{expected_value}'."
    )
    CLUSTER_DEFINITION_FLAGGED_VALUES = (
        "Container has outlier values which are non-fatal: {warning_reasons}"
    )


class ValidationWarningSolutionHintTemplate(Enum):
    VPC_PEERING_PEERING_NOT_READY = "Please try again in a moment."



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass

from dataclasses_json import dataclass_json


@dataclass_json
@dataclass
class LogEvent:
    timestamp: int
    message: str



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from datetime import date
from typing import Optional, Dict, Any

from fbpcp.entity.cloud_cost import CloudCost
from fbpcp.gateway.costexplorer import CostExplorerGateway
from fbpcp.service.billing import BillingService


class AWSBillingService(BillingService):
    def __init__(
        self,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        self.ce_gateway = CostExplorerGateway(access_key_id, access_key_data, config)

    def get_cost(
        self, start_date: date, end_date: date, region: Optional[str] = None
    ) -> CloudCost:
        """Get cost between start_date and end_date
        Keyword arguments:
        start_date: start date for cost
        end_date: end date for cost
        region: region name as optional filter for cost.
        """
        date_format = "%Y-%m-%d"
        return self.ce_gateway.get_cost(
            start_date.strftime(date_format),
            end_date.strftime(date_format),
            region,
        )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Type, TypeVar


T = TypeVar("T")
V = TypeVar("V")


def checked_cast(typ: Type[T], val: V) -> T:
    if not isinstance(val, typ):
        raise ValueError(f"Value was not of type {type!r}:\n{val!r}")
    return val



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Dict, List, Optional, Any, Final

import boto3
from fbpcp.decorator.error_handler import error_handler
from fbpcp.decorator.metrics import request_counter, duration_time, error_counter
from fbpcp.entity.cluster_instance import Cluster
from fbpcp.entity.container_definition import ContainerDefinition
from fbpcp.entity.container_instance import ContainerInstance
from fbpcp.error.pcp import PcpError
from fbpcp.gateway.aws import AWSGateway
from fbpcp.mapper.aws import (
    map_ecstask_to_containerinstance,
    map_esccluster_to_clusterinstance,
    map_ecstaskdefinition_to_containerdefinition,
)
from fbpcp.metrics.emitter import MetricsEmitter
from fbpcp.metrics.getter import MetricsGetter

METRICS_RUN_TASK_COUNT = "aws.ecs.run_task.count"
METRICS_RUN_TASK_ERROR_COUNT = "aws.ecs.run_task.error.count"
METRICS_RUN_TASK_DURATION = "aws.ecs.run_task.duration"


class ECSGateway(AWSGateway, MetricsGetter):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
        metrics: Optional[MetricsEmitter] = None,
    ) -> None:
        super().__init__(region, access_key_id, access_key_data, config)

        # pyre-ignore
        self.client = boto3.client("ecs", region_name=self.region, **self.config)
        self.metrics: Final[Optional[MetricsEmitter]] = metrics

    def has_metrics(self) -> bool:
        return self.metrics is not None

    def get_metrics(self) -> MetricsEmitter:
        if not self.metrics:
            raise PcpError("ECSGateway doesn't have metrics emitter")

        return self.metrics

    @error_counter(METRICS_RUN_TASK_ERROR_COUNT)
    @request_counter(METRICS_RUN_TASK_COUNT)
    @duration_time(METRICS_RUN_TASK_DURATION)
    @error_handler
    def run_task(
        self,
        task_definition: str,
        container: str,
        cmd: str,
        cluster: str,
        subnets: List[str],
        env_vars: Optional[Dict[str, str]] = None,
    ) -> ContainerInstance:
        environment = []
        if env_vars:
            environment = [
                {"name": env_name, "value": env_value}
                for env_name, env_value in env_vars.items()
            ]
        response = self.client.run_task(
            taskDefinition=task_definition,
            cluster=cluster,
            networkConfiguration={
                "awsvpcConfiguration": {
                    "subnets": subnets,
                    "assignPublicIp": "ENABLED",
                }
            },
            overrides={
                "containerOverrides": [
                    {
                        "name": container,
                        "command": [cmd],
                        "environment": environment,
                    }
                ]
            },
        )

        if not response["tasks"]:
            # common failures: https://docs.aws.amazon.com/AmazonECS/latest/developerguide/api_failures_messages.html
            failure = response["failures"][0]
            self.logger.warn(f"ECSGateway failed to create a task. Failure: {failure}")
            raise PcpError(f"ECS failure: reason: {failure['reason']}")

        return map_ecstask_to_containerinstance(response["tasks"][0])

    @error_handler
    def describe_tasks(self, cluster: str, tasks: List[str]) -> List[ContainerInstance]:
        response = self.client.describe_tasks(cluster=cluster, tasks=tasks)
        return [map_ecstask_to_containerinstance(task) for task in response["tasks"]]

    @error_handler
    def describe_task(self, cluster: str, task: str) -> ContainerInstance:
        return self.describe_tasks(cluster, [task])[0]

    @error_handler
    def list_tasks(self, cluster: str) -> List[str]:
        return self.client.list_tasks(cluster=cluster)["taskArns"]

    @error_handler
    def stop_task(self, cluster: str, task_id: str) -> None:
        self.client.stop_task(
            cluster=cluster,
            task=task_id,
        )

    @error_handler
    def describe_clusters(
        self,
        clusters: Optional[List[str]] = None,
        tags: Optional[Dict[str, str]] = None,
    ) -> List[Cluster]:
        if not clusters:
            clusters = self.list_clusters()
        response = self.client.describe_clusters(clusters=clusters, include=["TAGS"])
        cluster_instances = [
            map_esccluster_to_clusterinstance(cluster)
            for cluster in response["clusters"]
        ]
        if tags:
            return list(
                filter(
                    lambda cluster_instance: tags.items()
                    <= cluster_instance.tags.items(),
                    cluster_instances,
                )
            )
        return cluster_instances

    @error_handler
    def describe_cluster(self, cluster: str) -> Cluster:
        return self.describe_clusters(clusters=[cluster])[0]

    @error_handler
    def list_clusters(self) -> List[str]:
        return self.client.list_clusters()["clusterArns"]

    @error_handler
    def describe_task_definition(self, task_defination: str) -> ContainerDefinition:
        response = self.client.describe_task_definition(
            taskDefinition=task_defination, include=["TAGS"]
        )
        return map_ecstaskdefinition_to_containerdefinition(
            response["taskDefinition"], response["tags"]
        )

    @error_handler
    def list_task_definitions(self) -> List[str]:
        return self.client.list_task_definitions()["taskDefinitionArns"]

    @error_handler
    def describe_task_definitions(
        self,
        task_definations: Optional[List[str]] = None,
        tags: Optional[Dict[str, str]] = None,
    ) -> List[ContainerDefinition]:
        if not task_definations:
            task_definations = self.list_task_definitions()
        container_definitions = []
        for arn in task_definations:
            container_definition = self.describe_task_definition(arn)
            if tags is None or tags.items() <= container_definition.tags.items():
                container_definitions.append(container_definition)
        return container_definitions



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from dataclasses import dataclass
from typing import Optional

from fbpcp.entity.cluster_instance import Cluster
from fbpcp.entity.container_definition import ContainerDefinition


@dataclass
class PCECompute:
    region: str
    cluster: Optional[Cluster]
    container_definition: Optional[ContainerDefinition]



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass, field
from enum import Enum
from typing import Dict

from dataclasses_json import dataclass_json


class VpcState(Enum):
    UNKNOWN = "UNKNOWN"
    PENDING = "PENDING"
    AVAILABLE = "AVAILABLE"


@dataclass_json
@dataclass
class Vpc:
    vpc_id: str
    cidr: str
    state: VpcState = VpcState.UNKNOWN
    tags: Dict[str, str] = field(default_factory=dict)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass


@dataclass
class PackageInfo:
    package_name: str
    version: str
    last_modified: str
    package_size: int



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict


class PcpError(Exception):
    pass


class InvalidParameterError(PcpError):
    pass


class ThrottlingError(PcpError):
    pass



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# This is the repository path that OneDocker downloads binaries from
ONEDOCKER_REPOSITORY_PATH = "ONEDOCKER_REPOSITORY_PATH"

# This is the local path that the binaries reside
ONEDOCKER_EXE_PATH = "ONEDOCKER_EXE_PATH"



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from botocore.exceptions import ClientError
from fbpcp.error.pcp import InvalidParameterError
from fbpcp.error.pcp import PcpError
from fbpcp.error.pcp import ThrottlingError


# reference: https://boto3.amazonaws.com/v1/documentation/api/latest/guide/error-handling.html
def map_aws_error(error: ClientError) -> PcpError:
    code = error.response["Error"]["Code"]
    message = str(error)

    if code == "InvalidParameterException":
        return InvalidParameterError(message)

    if code == "ThrottlingException":
        return ThrottlingError(message)

    return PcpError(message)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.


from pce.validator.validator import main

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import re
from typing import Tuple


class S3Path:
    region: str
    bucket: str
    key: str

    def __init__(self, fileURL: str) -> None:
        self.region, self.bucket, self.key = self._get_region_bucket_key(fileURL)

    def __eq__(self, other: "S3Path") -> bool:
        return (
            self.region == other.region
            and self.bucket == other.bucket
            and self.key == other.key
        )

    # virtual host style url
    # https://bucket-name.s3.Region.amazonaws.com/key-name
    def _get_region_bucket_key(self, fileURL: str) -> Tuple[str, str, str]:
        match = re.search("^https?:/([^.]+).s3.([^.]+).amazonaws.com/(.*)$", fileURL)
        if not match:
            raise ValueError(f"Could not parse {fileURL} as an S3Path")
        bucket, region, key = (
            match.group(1).strip("/"),
            match.group(2),
            match.group(3).strip("/"),
        )
        return (region, bucket, key)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from dataclasses import dataclass
from decimal import Decimal
from typing import List


@dataclass
class CloudCostItem:
    service: str
    cost_amount: Decimal


@dataclass
class CloudCost:
    total_cost_amount: Decimal
    details: List[CloudCostItem]



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from dataclasses import dataclass
from typing import List, Optional

from fbpcp.entity.firewall_ruleset import FirewallRuleset
from fbpcp.entity.route_table import RouteTable
from fbpcp.entity.subnet import Subnet
from fbpcp.entity.vpc_instance import Vpc
from fbpcp.entity.vpc_peering import VpcPeering


@dataclass
class PCENetwork:
    region: str
    vpc: Optional[Vpc]
    subnets: List[Subnet]
    route_table: Optional[RouteTable]
    vpc_peering: Optional[VpcPeering]
    firewall_rulesets: List[FirewallRuleset]



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import abc
from typing import List

from fbpcp.entity.container_instance import ContainerInstance
from fbpcp.entity.log_event import LogEvent


class LogService(abc.ABC):
    @abc.abstractmethod
    def fetch(self, log_path: str, start_time: int) -> List[LogEvent]:
        pass

    @abc.abstractmethod
    def get_log_path(self, container_instance: ContainerInstance) -> str:
        pass



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import abc

from fbpcp.entity.mpc_game_config import MPCGameConfig


class MPCGameRepository(abc.ABC):
    @abc.abstractmethod
    def get_game(self, name: str) -> MPCGameConfig:
        pass



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import abc
from enum import Enum
from typing import List

from fbpcp.entity.file_information import FileInfo


class PathType(Enum):
    Local = 1
    S3 = 2
    GCS = 3


class StorageService(abc.ABC):
    @abc.abstractmethod
    def read(self, filename: str) -> str:
        pass

    @abc.abstractmethod
    def write(self, filename: str, data: str) -> None:
        pass

    @abc.abstractmethod
    def copy(self, source: str, destination: str) -> None:
        pass

    @abc.abstractmethod
    def file_exists(self, filename: str) -> bool:
        pass

    @staticmethod
    def path_type(filename: str) -> PathType:
        if filename.startswith("https:"):
            return PathType.S3

        return PathType.Local

    @abc.abstractmethod
    def get_file_size(self, filename: str) -> int:
        pass

    @abc.abstractmethod
    def get_file_info(self, filename: str) -> FileInfo:
        pass

    @abc.abstractmethod
    def list_folders(self, filename: str) -> List[str]:
        pass



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from pathlib import Path
from typing import Any, Dict

import yaml


def load(file_path: Path) -> Dict[str, Any]:
    with open(file_path) as stream:
        return yaml.safe_load(stream)


# pyre-ignore
def dump(data: Any, file_path: Path) -> None:
    with open(file_path, "w") as f:
        return yaml.dump(data, f)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from dataclasses import dataclass, field
from typing import Dict


@dataclass
class Subnet:
    id: str
    availability_zone: str
    tags: Dict[str, str] = field(default_factory=dict)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict


from functools import reduce
from typing import Dict, List, Optional


def convert_dict_to_list(
    target_dict: Dict[str, str], key: str, value: str
) -> List[Dict[str, List[str]]]:
    """
    Input examples:
        target_dict: {"k1": "v1", "k2": "v2"}
        key: "Name",
        value: "Values"
    Output examples:
        [
            {"Name": "k1", "Values": ["v1"]},
            {"Name": "k2", "Values": ["v2"]},
        ]
    """
    return reduce(
        lambda x, y: [*x, {key: y, value: [target_dict[y]]}], target_dict.keys(), []
    )


def convert_list_to_dict(
    target_list: List[Dict[str, str]], key: str, value: str
) -> Dict[str, str]:
    """
    Input examples:
        target_list:
        [
            {"Name": "k1", "Values": "v1"},
            {"Name": "k2", "Values": "v2"},
        ]
        key: "Name",
        value: "Value"
    Output examples: {"k1": "v1", "k2": "v2"}
    """
    if target_list is not None:
        return reduce(lambda x, y: {**x, **{y[key]: y[value]}}, target_list, {})
    else:
        return {}


def prepare_tags(tags: Dict[str, str]) -> Dict[str, str]:
    """
    Input examples: {"k1": "v1", "k2": "v2"}
    Output examples: {"tag:k1": "v1", "tag:k2", "v2"}
    """
    return reduce(lambda x, y: {**x, **{f"tag:{y}": tags[y]}}, tags.keys(), {})


def convert_vpc_tags_to_filter(
    tags: Optional[Dict[str, str]] = None, vpc_id: Optional[str] = None
) -> List[Dict[str, List[str]]]:
    """
    Input examples:
        tags [Optional]: {"k1": "v1", "k2": "v2"},
        vpc_id [Optional]: "vpc-id-ex"
    Output examples:
        [
            {"Name": "k1", "Values": ["v1"]},
            {"Name": "k2", "Values": ["v2"]},
            {"Name": "vpc-id", "Values": ["vpc-id-ex"]}
        ]
    """
    vpc_dict = {"vpc-id": vpc_id} if vpc_id else {}
    tags_dict = prepare_tags(tags) if tags else {}
    filter_dict = {**vpc_dict, **tags_dict}
    return convert_dict_to_list(filter_dict, "Name", "Values") if filter_dict else []



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import os
from os import path
from os.path import join, normpath, relpath
from typing import Any, Dict, Optional, List

from fbpcp.entity.file_information import FileInfo
from fbpcp.gateway.s3 import S3Gateway
from fbpcp.service.storage import PathType, StorageService
from fbpcp.util.s3path import S3Path


class S3StorageService(StorageService):
    def __init__(
        self,
        region: str = "us-west-1",
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        self.s3_gateway = S3Gateway(region, access_key_id, access_key_data, config)

    def read(self, filename: str) -> str:
        """Read a file data
        Keyword arguments:
        filename -- "https://bucket-name.s3.Region.amazonaws.com/key-name"
        """
        s3_path = S3Path(filename)
        return self.s3_gateway.get_object(s3_path.bucket, s3_path.key)

    def write(self, filename: str, data: str) -> None:
        """Write data into a file
        Keyword arguments:
        filename -- "https://bucket-name.s3.Region.amazonaws.com/key-name"`
        """
        s3_path = S3Path(filename)
        self.s3_gateway.put_object(s3_path.bucket, s3_path.key, data)

    def copy(self, source: str, destination: str, recursive: bool = False) -> None:
        """Move a file or folder between local storage and S3, as well as, S3 and S3
        Keyword arguments:
        source -- source file
        destination -- destination file
        recursive -- whether to recursively copy a folder
        """
        if StorageService.path_type(source) == PathType.Local:
            # from local to S3
            if StorageService.path_type(destination) == PathType.Local:
                raise ValueError("Both source and destination are local files")
            s3_path = S3Path(destination)
            if path.isdir(source):
                if not recursive:
                    raise ValueError(f"Source {source} is a folder. Use --recursive")
                self.upload_dir(source, s3_path.bucket, s3_path.key)
            else:
                self.s3_gateway.upload_file(source, s3_path.bucket, s3_path.key)
        else:
            source_s3_path = S3Path(source)
            if StorageService.path_type(destination) == PathType.S3:
                # from S3 to S3
                dest_s3_path = S3Path(destination)
                if source_s3_path == dest_s3_path:
                    raise ValueError(
                        f"Source {source} and destination {destination} are the same"
                    )

                if source.endswith("/"):
                    if not recursive:
                        raise ValueError(
                            f"Source {source} is a folder. Use --recursive"
                        )

                    self.copy_dir(
                        source_s3_path.bucket,
                        source_s3_path.key + "/",
                        dest_s3_path.bucket,
                        dest_s3_path.key,
                    )
                else:
                    self.s3_gateway.copy(
                        source_s3_path.bucket,
                        source_s3_path.key,
                        dest_s3_path.bucket,
                        dest_s3_path.key,
                    )
            else:
                # from S3 to local
                if source.endswith("/"):
                    if not recursive:
                        raise ValueError(
                            f"Source {source} is a folder. Use --recursive"
                        )
                    self.download_dir(
                        source_s3_path.bucket,
                        source_s3_path.key + "/",
                        destination,
                    )
                else:
                    self.s3_gateway.download_file(
                        source_s3_path.bucket, source_s3_path.key, destination
                    )

    def upload_dir(self, source: str, s3_path_bucket: str, s3_path_key: str) -> None:
        for root, dirs, files in os.walk(source):
            for file in files:
                local_path = join(root, file)
                destination_path = s3_path_key + "/" + relpath(local_path, source)

                self.s3_gateway.upload_file(
                    local_path,
                    s3_path_bucket,
                    destination_path,
                )
            for dir in dirs:
                local_path = join(root, dir)
                destination_path = s3_path_key + "/" + relpath(local_path, source)

                self.s3_gateway.put_object(
                    s3_path_bucket,
                    destination_path + "/",
                    "",
                )

    def download_dir(
        self, s3_path_bucket: str, s3_path_key: str, destination: str
    ) -> None:
        if not self.s3_gateway.object_exists(s3_path_bucket, s3_path_key):
            raise ValueError(
                f"Key {s3_path_key} does not exist in bucket {s3_path_bucket}"
            )
        keys = self.s3_gateway.list_object2(s3_path_bucket, s3_path_key)
        for key in keys:
            local_path = normpath(destination + "/" + key[len(s3_path_key) :])
            if key.endswith("/"):
                if not path.exists(local_path):
                    os.makedirs(local_path)
            else:
                self.s3_gateway.download_file(s3_path_bucket, key, local_path)

    def copy_dir(
        self,
        source_bucket: str,
        source_key: str,
        destination_bucket: str,
        destination_key: str,
    ) -> None:
        if not self.s3_gateway.object_exists(source_bucket, source_key):
            raise ValueError(
                f"Key {source_key} does not exist in bucket {source_bucket}"
            )
        keys = self.s3_gateway.list_object2(source_bucket, source_key)
        for key in keys:
            destination_path = destination_key + "/" + key[len(source_key) :]
            if key.endswith("/"):
                self.s3_gateway.put_object(
                    source_bucket,
                    destination_path,
                    "",
                )
            else:
                self.s3_gateway.copy(
                    source_bucket,
                    key,
                    destination_bucket,
                    destination_path,
                )

    def delete(self, filename: str) -> None:
        """Delete an s3 file
        Keyword arguments:
        filename -- the s3 file to be deleted
        """
        if StorageService.path_type(filename) == PathType.S3:
            s3_path = S3Path(filename)
            self.s3_gateway.delete_object(s3_path.bucket, s3_path.key)
        else:
            raise ValueError("The file is not an s3 file")

    def file_exists(self, filename: str) -> bool:
        if StorageService.path_type(filename) == PathType.S3:
            s3_path = S3Path(filename)
            return self.s3_gateway.object_exists(s3_path.bucket, s3_path.key)
        else:
            raise ValueError(f"File {filename} is not an S3 filepath")

    def get_file_info(self, filename: str) -> FileInfo:
        """Show file information (last modified time, type and size)
        Keyword arguments:
        filename -- the s3 file to be shown
        """
        s3_path = S3Path(filename)
        file_info_dict = self.s3_gateway.get_object_info(s3_path.bucket, s3_path.key)
        return FileInfo(
            file_name=filename,
            last_modified=file_info_dict.get("LastModified").ctime(),
            file_size=file_info_dict.get("ContentLength"),
        )

    def get_file_size(self, filename: str) -> int:
        s3_path = S3Path(filename)
        return self.s3_gateway.get_object_size(s3_path.bucket, s3_path.key)

    def list_folders(self, filename: str) -> List[str]:
        s3_path = S3Path(filename)
        return self.s3_gateway.list_folders(s3_path.bucket, s3_path.key)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from decimal import Decimal
from typing import Any, Dict, List

from fbpcp.entity.cloud_cost import CloudCost, CloudCostItem
from fbpcp.entity.cluster_instance import Cluster, ClusterStatus
from fbpcp.entity.container_definition import ContainerDefinition
from fbpcp.entity.container_instance import ContainerInstance, ContainerInstanceStatus
from fbpcp.entity.firewall_ruleset import FirewallRule, FirewallRuleset
from fbpcp.entity.route_table import (
    RouteTable,
    Route,
    RouteTarget,
    RouteTargetType,
    RouteState,
)
from fbpcp.entity.subnet import Subnet
from fbpcp.entity.vpc_instance import Vpc, VpcState
from fbpcp.entity.vpc_peering import VpcPeering, VpcPeeringRole, VpcPeeringState
from fbpcp.util.aws import convert_list_to_dict


def map_ecstask_to_containerinstance(task: Dict[str, Any]) -> ContainerInstance:
    container = task["containers"][0]
    ip_v4 = (
        container["networkInterfaces"][0]["privateIpv4Address"]
        if len(container["networkInterfaces"]) > 0
        else None
    )

    status = container["lastStatus"]
    if status == "RUNNING":
        status = ContainerInstanceStatus.STARTED
    elif status == "STOPPED":
        if container.get("exitCode") == 0:
            status = ContainerInstanceStatus.COMPLETED
        else:
            status = ContainerInstanceStatus.FAILED
    else:
        status = ContainerInstanceStatus.UNKNOWN

    return ContainerInstance(task["taskArn"], ip_v4, status)


def map_esccluster_to_clusterinstance(cluster: Dict[str, Any]) -> Cluster:
    status = cluster["status"]
    if status == "ACTIVE":
        status = ClusterStatus.ACTIVE
    elif status == "INACTIVE":
        status = ClusterStatus.INACTIVE
    else:
        status = ClusterStatus.UNKNOWN

    tags = convert_list_to_dict(cluster["tags"], "key", "value")
    return Cluster(
        cluster_arn=cluster["clusterArn"],
        cluster_name=cluster["clusterName"],
        pending_tasks=cluster["pendingTasksCount"],
        running_tasks=cluster["runningTasksCount"],
        status=status,
        tags=tags,
    )


def map_ec2vpc_to_vpcinstance(vpc: Dict[str, Any]) -> Vpc:
    state = vpc["State"]
    if state == "pending":
        state = VpcState.PENDING
    elif state == "available":
        state = VpcState.AVAILABLE
    else:
        state = VpcState.UNKNOWN

    vpc_id = vpc["VpcId"]
    cidr_block = vpc["CidrBlock"]
    # some vpc instances don't have any tags
    tags = convert_list_to_dict(vpc.get("Tags"), "Key", "Value")

    # TODO add implementation to get the firewall_ruleset
    return Vpc(vpc_id, cidr_block, state, tags)


def map_ec2subnet_to_subnet(subnet: Dict[str, Any]) -> Subnet:
    availability_zone = subnet["AvailabilityZone"]
    subnet_id = subnet["SubnetId"]
    tags = convert_list_to_dict(subnet.get("Tags"), "Key", "Value")
    return Subnet(subnet_id, availability_zone, tags)


def map_cecost_to_cloud_cost(cost_by_date: List[Dict[str, Any]]) -> CloudCost:
    total_cost_amount = Decimal(0)
    cost_items = {}
    for daily_result in cost_by_date:
        for group_result in daily_result.get("Groups"):
            amount = Decimal(group_result["Metrics"]["UnblendedCost"]["Amount"])
            total_cost_amount += amount
            cost_items[group_result["Keys"][0]] = (
                cost_items.get(group_result["Keys"][0], 0) + amount
            )

    return CloudCost(
        total_cost_amount=total_cost_amount,
        details=[
            CloudCostItem(
                service=service,
                cost_amount=amount,
            )
            for service, amount in cost_items.items()
        ],
    )


def map_ec2route_to_route(route: Dict[str, Any]) -> Route:
    destination_cidr = route["DestinationCidrBlock"]
    route_target_type = RouteTargetType.OTHER
    route_target_id = ""
    if "VpcPeeringConnectionId" in route:
        route_target_type = RouteTargetType.VPC_PEERING
        route_target_id = route["VpcPeeringConnectionId"]
    elif "GatewayId" in route:
        route_target_type = RouteTargetType.INTERNET
        route_target_id = route["GatewayId"]
    state = RouteState.UNKNOWN
    if route["State"] == "active":
        state = RouteState.ACTIVE
    route_target = RouteTarget(route_target_id, route_target_type)
    return Route(destination_cidr, route_target, state)


def map_ec2routetable_to_routetable(route_table: Dict[str, Any]) -> RouteTable:
    route_table_id = route_table["RouteTableId"]
    routes = [map_ec2route_to_route(route) for route in route_table["Routes"]]
    vpc_id = route_table["VpcId"]
    tags = convert_list_to_dict(route_table.get("Tags"), "Key", "Value")
    return RouteTable(route_table_id, routes, vpc_id, tags)


def map_ec2ippermission_to_firewallrule(ip_permission: Dict[str, Any]) -> FirewallRule:
    ip_protocol = ip_permission["IpProtocol"]
    from_port = ip_permission.get("FromPort", -1)
    to_port = ip_permission.get("ToPort", -1)
    ip_range = ip_permission["IpRanges"][0]
    cidr = ip_range["CidrIp"]
    return FirewallRule(from_port, to_port, ip_protocol, cidr)


def map_ec2securitygroup_to_firewallruleset(
    security_group: Dict[str, Any]
) -> FirewallRuleset:
    id = security_group["GroupId"]
    vpc_id = security_group["VpcId"]
    tags = convert_list_to_dict(security_group.get("Tags"), "Key", "Value")
    ingress = [
        map_ec2ippermission_to_firewallrule(ip_permission)
        for ip_permission in security_group["IpPermissions"]
    ]
    egress = [
        map_ec2ippermission_to_firewallrule(ip_permission)
        for ip_permission in security_group["IpPermissionsEgress"]
    ]
    return FirewallRuleset(id, vpc_id, ingress, egress, tags)


def map_ec2vpcpeering_to_vpcpeering(
    vpc_peering: Dict[str, Any], vpc_id: str
) -> VpcPeering:
    id = vpc_peering["VpcPeeringConnectionId"]
    status = VpcPeeringState.NOT_READY
    status_code = vpc_peering["Status"]["Code"]
    if status_code == "active":
        status = VpcPeeringState.ACTIVE
    elif status_code == "pending-acceptance":
        status = VpcPeeringState.PENDING_ACCEPTANCE
    elif status_code == "rejected":
        status = VpcPeeringState.REJECTED
    requester_vpc_id = vpc_peering["RequesterVpcInfo"]["VpcId"]
    accepter_vpc_id = vpc_peering["AccepterVpcInfo"]["VpcId"]
    role = (
        VpcPeeringRole.REQUESTER
        if requester_vpc_id == vpc_id
        else VpcPeeringRole.ACCEPTER
    )
    tags = convert_list_to_dict(vpc_peering.get("Tags"), "Key", "Value")
    return VpcPeering(id, status, role, requester_vpc_id, accepter_vpc_id, tags)


def map_ecstaskdefinition_to_containerdefinition(
    task_definition: Dict[str, Any],
    tag_list: List[Dict[str, str]],
) -> ContainerDefinition:
    task_definition_id = task_definition["taskDefinitionArn"]
    container_definition = task_definition["containerDefinitions"][0]
    image = container_definition["image"]
    cpu = container_definition.get("cpu", task_definition.get("cpu"))
    memory = container_definition.get("memory", task_definition.get("memory"))
    entry_point = container_definition.get("entryPoint", [])
    environment = convert_list_to_dict(
        container_definition["environment"], "name", "value"
    )
    task_role_id = task_definition.get("taskRoleArn", None)
    tags = convert_list_to_dict(tag_list, "key", "value")
    return ContainerDefinition(
        task_definition_id,
        image,
        cpu,
        memory,
        entry_point,
        environment,
        task_role_id,
        tags,
    )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from dataclasses import dataclass, field
from enum import Enum
from typing import Dict, List


class RouteTargetType(Enum):
    OTHER = "OTHER"
    INTERNET = "INTERNET"
    VPC_PEERING = "VPC_PEERING"


class RouteState(Enum):
    UNKNOWN = "UNKNOWN"
    ACTIVE = "ACTIVE"


@dataclass
class RouteTarget:
    route_target_id: str
    route_target_type: RouteTargetType


@dataclass
class Route:
    destination_cidr_block: str
    route_target: RouteTarget
    state: RouteState


@dataclass
class RouteTable:
    id: str
    routes: List[Route]
    vpc_id: str
    tags: Dict[str, str] = field(default_factory=dict)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import abc


class MetricsEmitter(abc.ABC):
    @abc.abstractmethod
    def count(
        self,
        name: str,
        value: int,
    ) -> None:
        pass

    @abc.abstractmethod
    def gauge(
        self,
        name: str,
        value: int,
    ) -> None:
        pass



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import ipaddress
from collections import defaultdict
from dataclasses import dataclass
from enum import Enum
from typing import Optional, Dict, Any, List, Tuple

from fbpcp.entity.firewall_ruleset import FirewallRuleset
from fbpcp.entity.pce import PCE
from fbpcp.entity.route_table import Route, RouteState, RouteTargetType
from fbpcp.entity.vpc_instance import Vpc
from fbpcp.entity.vpc_peering import VpcPeeringState
from pce.gateway.ec2 import EC2Gateway
from pce.validator.message_templates import (
    ValidationErrorDescriptionTemplate,
    ValidationErrorSolutionHintTemplate,
    ValidationWarningDescriptionTemplate,
    ValidationWarningSolutionHintTemplate,
)
from pce.validator.pce_standard_constants import (
    FIREWALL_RULE_INITIAL_PORT,
    FIREWALL_RULE_FINAL_PORT,
    CONTAINER_CPU,
    CONTAINER_MEMORY,
    CONTAINER_IMAGE,
)


class ValidationResultCode(Enum):
    SUCCESS = "SUCCESS"
    ERROR = "ERROR"
    WARNING = "WARNING"


@dataclass
class ValidationResult:
    validation_result_code: ValidationResultCode = ValidationResultCode.ERROR
    description: Optional[str] = None
    solution_hint: Optional[str] = None


class ClusterResourceType(Enum):
    CPU = "cpu"
    MEMORY = "memory"
    IMAGE = "image"


class ValidationSuite:
    def __init__(
        self,
        region: str,
        key_id: str,
        key_data: str,
        config: Optional[Dict[str, Any]] = None,
        ec2_gateway: Optional[EC2Gateway] = None,
    ) -> None:
        self.ec2_gateway: EC2Gateway = ec2_gateway or EC2Gateway(
            region, key_id, key_data, config
        )

    def validate_private_cidr(self, pce: PCE) -> ValidationResult:
        vpc = pce.pce_network.vpc
        if not vpc:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.VPC_PEERING_NO_VPC.value,
            )
        is_valid = ipaddress.ip_network(vpc.cidr).is_private
        return (
            ValidationResult(ValidationResultCode.SUCCESS)
            if is_valid
            else ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.NON_PRIVATE_VPC_CIDR.value.format(
                    vpc_cidr=vpc.vpc_id
                ),
                ValidationErrorSolutionHintTemplate.NON_PRIVATE_VPC_CIDR.value,
            )
        )

    def validate_vpc_peering(self, pce: PCE) -> ValidationResult:
        vpc_peering = pce.pce_network.vpc_peering
        if not vpc_peering:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.VPC_PEERING_NO_VPC_PEERING.value,
            )

        state_results = defaultdict(
            lambda: ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.UNKNOWN.value,
            ),
            {
                VpcPeeringState.ACTIVE: ValidationResult(ValidationResultCode.SUCCESS),
                VpcPeeringState.PENDING_ACCEPTANCE: ValidationResult(
                    ValidationResultCode.WARNING,
                    ValidationWarningDescriptionTemplate.VPC_PEERING_PEERING_NOT_READY.value,
                    ValidationWarningSolutionHintTemplate.VPC_PEERING_PEERING_NOT_READY.value,
                ),
            },
        )

        return state_results[vpc_peering.status]

    def _check_inbound_peer_route_allowed(
        self,
        peer_routes: List[Route],
        firewall_rulesets: List[FirewallRuleset],
        vpc: Vpc,
    ) -> Tuple[List[str], List[str]]:
        """
        For each peer route check if there is an inbound rule for it. If there is, make sure the port range of the rule is within PCE standard range. The results are returned in 2 lists of messages, one for errors and one for warnings.
        """
        error_reasons = []
        warning_reasons = []

        for peer_route in peer_routes:
            peer_cidr = peer_route.destination_cidr_block
            peer_rule_found = False
            for fr in firewall_rulesets:
                for fri in fr.ingress:
                    if not ipaddress.ip_network(fri.cidr).overlaps(
                        ipaddress.ip_network(peer_cidr)
                    ):
                        continue
                    peer_rule_found = True
                    if (
                        FIREWALL_RULE_INITIAL_PORT < fri.from_port
                        or FIREWALL_RULE_FINAL_PORT > fri.to_port
                    ):
                        error_reasons.append(
                            ValidationErrorDescriptionTemplate.FIREWALL_CIDR_CANT_CONTAIN_EXPECTED_RANGE.value.format(
                                fr_vpc_id=fr.vpc_id,
                                fri_cidr=fri.cidr,
                                fri_from_port=fri.from_port,
                                fri_to_port=fri.to_port,
                            ),
                        )
                    elif (
                        FIREWALL_RULE_INITIAL_PORT > fri.from_port
                        or FIREWALL_RULE_FINAL_PORT < fri.to_port
                    ):
                        warning_reasons.append(
                            ValidationWarningDescriptionTemplate.FIREWALL_CIDR_EXCEED_EXPECTED_RANGE.value.format(
                                fr_vpc_id=fr.vpc_id,
                                fri_cidr=fri.cidr,
                                fri_from_port=fri.from_port,
                                fri_to_port=fri.to_port,
                            )
                        )
            if not peer_rule_found:
                error_reasons.append(
                    ValidationErrorDescriptionTemplate.FIREWALL_CIDR_NOT_OVERLAPS_VPC.value.format(
                        peer_target_id=peer_route.route_target.route_target_id,
                        vpc_id=vpc.vpc_id,
                        vpc_cidr=vpc.cidr,
                    )
                )

        return error_reasons, warning_reasons

    def validate_firewall(self, pce: PCE) -> ValidationResult:
        """
        Check that among inbound traffic from the peers is allowed i.e. there is a rule whose CIDR is overlapped by any firewall rule.
        """
        vpc = pce.pce_network.vpc
        if not vpc:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.VPC_PEERING_NO_VPC.value,
            )
        vpc_cidr = vpc.cidr
        if not vpc_cidr:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.VPC_PEERING_NO_VPC_CIDR.value,
            )
        # Don't bother in validating route table if there are no firewall rules
        firewall_rulesets = pce.pce_network.firewall_rulesets
        if not firewall_rulesets:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.FIREWALL_RULES_NOT_FOUND.value.format(
                    pce_id=vpc.tags["pce:pce-id"]
                ),
            )
        route_table = pce.pce_network.route_table
        if not route_table:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.VPC_PEERING_NO_ROUTE_TABLE.value,
            )
        peer_routes = [
            r
            for r in (route_table.routes if route_table else [])
            if r.route_target.route_target_type == RouteTargetType.VPC_PEERING
        ]
        if not peer_routes:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.FIREWALL_PEER_ROUTE_NOT_SET.value,
            )

        error_reasons, warning_reasons = self._check_inbound_peer_route_allowed(
            peer_routes, firewall_rulesets, vpc
        )

        if error_reasons:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.FIREWALL_INVALID_RULESETS.value.format(
                    error_reasons=";".join(error_reasons)
                ),
                ValidationErrorSolutionHintTemplate.FIREWALL_INVALID_RULESETS.value,
            )
        if warning_reasons:
            return ValidationResult(
                ValidationResultCode.WARNING,
                ValidationWarningDescriptionTemplate.FIREWALL_FLAGGED_RULESETS.value.format(
                    warning_reasons=";".join(warning_reasons)
                ),
            )

        return ValidationResult(ValidationResultCode.SUCCESS)

    def validate_route_table(self, pce: PCE) -> ValidationResult:
        """
        Make sure there is a an entry in the route table for the VPC peer and that it is active
        """
        vpc = pce.pce_network.vpc
        if not vpc:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.VPC_PEERING_NO_VPC.value,
            )
        route_table = pce.pce_network.route_table
        if not route_table:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.VPC_PEERING_NO_ROUTE_TABLE.value,
            )

        is_valid = any(
            r.state == RouteState.ACTIVE
            and r.route_target.route_target_type == RouteTargetType.VPC_PEERING
            for r in (route_table.routes if route_table else [])
        )

        return (
            ValidationResult(ValidationResultCode.SUCCESS)
            if is_valid
            else ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.ROUTE_TABLE_VPC_PEERING_MISSING.value,
                ValidationErrorSolutionHintTemplate.ROUTE_TABLE_VPC_PEERING_MISSING.value,
            )
        )

    # Check that all subnets make use of every AZ in the VPC region
    def validate_subnets(self, pce: PCE) -> ValidationResult:
        region_azs = self.ec2_gateway.describe_availability_zones()
        is_valid = len(set(region_azs)) == len(
            {s.availability_zone for s in pce.pce_network.subnets}
        )
        return (
            ValidationResult(ValidationResultCode.SUCCESS)
            if is_valid
            else ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.NOT_ALL_AZ_USED.value,
                ValidationErrorSolutionHintTemplate.NOT_ALL_AZ_USED.value.format(
                    region=pce.pce_network.region, azs=",".join(region_azs)
                ),
            )
        )

    # Check CPU, memory and image name values are according to the standard
    def validate_cluster_definition(self, pce: PCE) -> ValidationResult:
        c = pce.pce_compute.container_definition
        if not c:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.CLUSTER_DEFINITION_NOT_SET.value,
            )
        message_by_level = defaultdict(list)
        for resource, expected_value, level in (
            (
                ClusterResourceType.CPU,
                CONTAINER_CPU,
                ValidationResultCode.ERROR,
            ),
            (
                ClusterResourceType.MEMORY,
                CONTAINER_MEMORY,
                ValidationResultCode.ERROR,
            ),
            (
                ClusterResourceType.IMAGE,
                CONTAINER_IMAGE,
                ValidationResultCode.WARNING,
            ),
        ):
            value = getattr(c, resource.value)
            if value != expected_value:
                message_template = (
                    ValidationErrorDescriptionTemplate.CLUSTER_DEFINITION_WRONG_VALUE.value
                    if ValidationResultCode.ERROR == level
                    else ValidationWarningDescriptionTemplate.CLUSTER_DEFINITION_FLAGGED_VALUE.value
                )
                message = message_template.format(
                    resource_name=resource.value.title(),
                    value=value,
                    expected_value=expected_value,
                )
                message_by_level[level].append(message)

        if message_by_level[ValidationResultCode.ERROR]:
            return ValidationResult(
                ValidationResultCode.ERROR,
                ValidationErrorDescriptionTemplate.CLUSTER_DEFINITION_WRONG_VALUES.value.format(
                    error_reasons=",".join(message_by_level[ValidationResultCode.ERROR])
                ),
                ValidationErrorSolutionHintTemplate.CLUSTER_DEFINITION_WRONG_VALUES.value,
            )
        elif message_by_level[ValidationResultCode.WARNING]:
            return ValidationResult(
                ValidationResultCode.WARNING,
                ValidationWarningDescriptionTemplate.CLUSTER_DEFINITION_FLAGGED_VALUES.value.format(
                    warning_reasons=",".join(
                        message_by_level[ValidationResultCode.WARNING]
                    )
                ),
            )
        return ValidationResult(ValidationResultCode.SUCCESS)

    def validate_network_and_compute(self, pce: PCE) -> List[ValidationResult]:
        """
        Execute all existing validations returning warnings and errors encapsulated in `ValidationResult` objects
        """
        return [
            vr
            for vr in [
                validation_function(pce)
                for validation_function in [
                    self.validate_private_cidr,
                    self.validate_vpc_peering,
                    self.validate_firewall,
                    self.validate_route_table,
                    self.validate_subnets,
                    self.validate_cluster_definition,
                ]
            ]
            if ValidationResultCode.SUCCESS != vr.validation_result_code
        ]

    @classmethod
    def summarize_errors(cls, validation_results: List[ValidationResult]) -> str:
        results_by_code = defaultdict(list)
        for result in validation_results:
            results_by_code[result.validation_result_code].append(result)
        return "\n".join(
            [
                f"{code}:\n\t"
                + "\n\t".join(
                    [
                        f"{res.description}{f' {res.solution_hint}' if res.solution_hint else ''}"
                        for res in results
                    ]
                )
                for code, results in results_by_code.items()
            ]
        )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import asyncio
import logging
from typing import Dict, List, Optional, Final

from fbpcp.decorator.metrics import request_counter, duration_time, error_counter
from fbpcp.entity.container_instance import ContainerInstance, ContainerInstanceStatus
from fbpcp.error.pcp import PcpError
from fbpcp.metrics.emitter import MetricsEmitter
from fbpcp.metrics.getter import MetricsGetter
from fbpcp.service.container import ContainerService
from fbpcp.util.arg_builder import build_cmd_args
from fbpcp.util.typing import checked_cast


ONEDOCKER_CMD_PREFIX = (
    # patternlint-disable-next-line f-string-may-be-missing-leading-f
    "python3.8 -m onedocker.script.runner {package_name} {runner_args}"
)

DEFAULT_BINARY_VERSION = "latest"

METRICS_CONTAINER_COUNT = "onedocker.container.count"
METRICS_START_CONTAINERS_COUNT = "onedocker.start_containers.count"
METRICS_START_CONTAINERS_ERROR_COUNT = "onedocker.start_containers.error.count"
METRICS_START_CONTAINERS_DURATION = "onedocker.start_containers.duration"


class OneDockerService(MetricsGetter):
    """OneDockerService is responsible for executing a package(binary) in a container on Cloud"""

    def __init__(
        self,
        container_svc: ContainerService,
        task_definition: Optional[str] = None,
        metrics: Optional[MetricsEmitter] = None,
    ) -> None:
        """Constructor of OneDockerService
        container_svc -- service to spawn container instances
        task_definition -- container definition to spawn container instances
        metrics -- metrics emitter to emit metrics
        """
        if container_svc is None:
            raise ValueError(f"Dependency is missing. container_svc={container_svc}, ")

        self.container_svc = container_svc
        self.task_definition = task_definition
        self.metrics: Final[Optional[MetricsEmitter]] = metrics
        self.logger: logging.Logger = logging.getLogger(__name__)

    def has_metrics(self) -> bool:
        return self.metrics is not None

    def get_metrics(self) -> MetricsEmitter:
        if not self.metrics:
            raise PcpError("OneDocker doesn't have metrics emitter")

        return self.metrics

    def start_container(
        self,
        package_name: str,
        task_definition: Optional[str] = None,
        version: str = DEFAULT_BINARY_VERSION,
        cmd_args: str = "",
        env_vars: Optional[Dict[str, str]] = None,
        timeout: Optional[int] = None,
        tag: Optional[str] = None,
    ) -> ContainerInstance:
        return self.start_containers(
            package_name,
            task_definition,
            version,
            [cmd_args] if cmd_args else None,
            env_vars,
            timeout,
            tag,
        )[0]

    @error_counter(METRICS_START_CONTAINERS_ERROR_COUNT)
    @request_counter(METRICS_START_CONTAINERS_COUNT)
    @duration_time(METRICS_START_CONTAINERS_DURATION)
    def start_containers(
        self,
        package_name: str,
        task_definition: Optional[str] = None,
        version: str = DEFAULT_BINARY_VERSION,
        cmd_args_list: Optional[List[str]] = None,
        env_vars: Optional[Dict[str, str]] = None,
        timeout: Optional[int] = None,
        tag: Optional[str] = None,
    ) -> List[ContainerInstance]:
        """Spin up one container per element in input command list.
        task_definition -- if specified, overrides OneDockerService's task definition
                           when starting this container
        """
        if not cmd_args_list:
            raise ValueError("Command Argument List shouldn't be None or Empty")

        cmds = [
            self._get_cmd(package_name, version, cmd_args, timeout)
            for cmd_args in cmd_args_list
        ]

        self.logger.info("Spinning up container instances")

        task_definition = task_definition or self.task_definition
        if not task_definition:
            raise ValueError(
                "task definition should be specified when spinning up containers"
            )
        containers = self.container_svc.create_instances(
            task_definition,
            cmds,
            env_vars,
        )

        if self.metrics:
            name = f"{METRICS_CONTAINER_COUNT}.{self.container_svc.get_region()}.{self.container_svc.get_cluster()}"
            name = f"{METRICS_CONTAINER_COUNT}.{tag}" if tag else name
            self.metrics.count(name, len(containers))
        return containers

    async def start_containers_async(
        self,
        package_name: str,
        task_definition: Optional[str] = None,
        version: str = DEFAULT_BINARY_VERSION,
        cmd_args_list: Optional[List[str]] = None,
        env_vars: Optional[Dict[str, str]] = None,
        timeout: Optional[int] = None,
        tag: Optional[str] = None,
    ) -> List[ContainerInstance]:
        containers = self.start_containers(
            package_name=package_name,
            task_definition=task_definition,
            version=version,
            cmd_args_list=cmd_args_list,
            env_vars=env_vars,
            timeout=timeout,
            tag=tag,
        )
        self.logger.info(
            f"OneDockerService is waiting for {len(containers)} container(s) to become started"
        )
        return await self.wait_for_pending_containers(
            [container.instance_id for container in containers]
        )

    async def wait_for_pending_containers(
        self, container_ids: List[str]
    ) -> List[ContainerInstance]:
        tasks = [
            asyncio.create_task(self.wait_for_pending_container(container_id))
            for container_id in container_ids
        ]
        res = await asyncio.gather(*tasks)
        return [checked_cast(ContainerInstance, container) for container in res]

    async def wait_for_pending_container(self, container_id: str) -> ContainerInstance:
        updated_container = self.get_containers([container_id])[0]
        while updated_container.status is ContainerInstanceStatus.UNKNOWN:
            await asyncio.sleep(1)
            updated_container = self.get_containers([container_id])[0]
        return updated_container

    def stop_containers(self, containers: List[str]) -> List[Optional[PcpError]]:
        return self.container_svc.cancel_instances(containers)

    def get_containers(self, instance_ids: List[str]) -> List[ContainerInstance]:
        return self.container_svc.get_instances(instance_ids)

    def _get_exe_name(self, package_name: str) -> str:
        return package_name.split("/")[1]

    def _get_cmd(
        self,
        package_name: str,
        version: str = DEFAULT_BINARY_VERSION,
        cmd_args: Optional[str] = None,
        timeout: Optional[int] = None,
    ) -> str:
        runner_args = build_cmd_args(
            exe_args=cmd_args,
            version=version,
            timeout=timeout,
        )
        return ONEDOCKER_CMD_PREFIX.format(
            package_name=package_name,
            runner_args=runner_args,
        ).strip()



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from dataclasses import dataclass

from fbpcp.entity.pce_compute import PCECompute
from fbpcp.entity.pce_network import PCENetwork


@dataclass
class PCE:
    pce_id: str
    region: str
    pce_network: PCENetwork
    pce_compute: PCECompute



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import logging
from typing import Any, Dict, Optional


class AWSGateway:
    def __init__(
        self,
        region: Optional[str] = None,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        self.logger: logging.Logger = logging.getLogger(__name__)
        self.region = region
        self.config: Dict[str, Any] = config or {}

        if access_key_id is not None:
            self.config["aws_access_key_id"] = access_key_id

        if access_key_data is not None:
            self.config["aws_secret_access_key"] = access_key_data



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from importlib import import_module
from typing import Any


# pyre-ignore
def get_class(class_path: str) -> Any:
    module_name, class_name = class_path.rsplit(".", 1)
    module = import_module(module_name)

    return getattr(module, class_name)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.


"""
CLI for running an executable in OneDocker containers.


Usage:
    onedocker-runner <package_name> --version=<version> [options]

Options:
    -h --help                           Show this help
    --repository_path=<repository_path> OneDocker repository path where the executables are downloaded from. No download when "LOCAL" repository is specified.
    --exe_path=<exe_path>               The local path where the executables are downloaded to.
    --exe_args=<exe_args>               The arguments the executable will use.
    --timeout=<timeout>                 Set timeout (in sec) to kill the task.
    --log_path=<path>                   Override the default path where logs are saved.
    --verbose                           Set logging level to DEBUG.
"""

import logging
import os
import subprocess
import sys
from pathlib import Path
from shlex import join, split
from typing import Any, Optional

import psutil
import schema
from docopt import docopt
from fbpcp.service.storage_s3 import S3StorageService
from fbpcp.util.s3path import S3Path
from onedocker.common.env import ONEDOCKER_EXE_PATH, ONEDOCKER_REPOSITORY_PATH
from onedocker.common.util import run_cmd


# The default OneDocker repository path on S3
DEFAULT_REPOSITORY_PATH = (
    "https://one-docker-repository-prod.s3.us-west-2.amazonaws.com/"
)

# The default path in the Docker image that is going to host the executables
DEFAULT_EXE_FOLDER = "/root/onedocker/package/"

# the default version of the binary
DEFAULT_BINARY_VERSION = "latest"

logger = None


def _prepare_executable(
    repository_path: str,
    exe_path: str,
    package_name: str,
    version: str,
) -> str:
    # download executable from s3
    if repository_path.upper() != "LOCAL":
        _download_executables(repository_path, package_name, version)
    else:
        logger.info("Local repository, skip download ...")

    # grant execute permission to the downloaded executable file
    exe_name = _parse_package_name(package_name)

    executable = f"{exe_path}{exe_name}"
    os.chmod(executable, 0o755)
    return executable


def _run_executable(
    executable: str, timeout: int, exe_args: Optional[str] = None
) -> None:
    # run execution cmd
    cmd = _build_cmd(executable, exe_args)
    logger.info(f"Running cmd: {cmd} ...")
    net_start: Any = psutil.net_io_counters()

    return_code = run_cmd(cmd, timeout)
    if return_code != 0:
        logger.error(f"Subprocess returned non-zero return code: {return_code}")

    net_end: Any = psutil.net_io_counters()
    logger.info(
        f"Net usage: {net_end.bytes_sent - net_start.bytes_sent} bytes sent, {net_end.bytes_recv - net_start.bytes_recv} bytes received"
    )

    sys.exit(return_code)


def _run_package(
    repository_path: str,
    exe_path: str,
    package_name: str,
    version: str,
    timeout: int,
    exe_args: Optional[str] = None,
) -> None:
    logger.info(f"Starting to run {package_name}, version: {version}")
    try:
        executable = _prepare_executable(
            repository_path=repository_path,
            exe_path=exe_path,
            package_name=package_name,
            version=version,
        )
    except Exception as err:
        logger.error(
            f"An error was raised while preparing {package_name}:{version} from {repository_path}, error: {err}",
            exc_info=True,
        )
        sys.exit(1)

    try:
        _run_executable(
            executable=executable,
            timeout=timeout,
            exe_args=exe_args,
        )
    except subprocess.TimeoutExpired:
        logger.error(
            f"{timeout} seconds have passed. Now exiting the program....",
            exc_info=True,
        )
        sys.exit(1)
    except InterruptedError:
        logger.error(
            "Receive abort command from user, Now exiting the program....",
            exc_info=True,
        )
        sys.exit(1)
    except Exception as err:
        logger.error(
            f"An error was raised while running {package_name}, error: {err}",
            exc_info=True,
        )
        sys.exit(1)


def _build_cmd(executable: str, exe_args: Optional[str]) -> str:
    args_list = split(exe_args) if exe_args else []
    args_list.insert(0, executable)
    return join(args_list)


def _download_executables(
    repository_path: str,
    package_name: str,
    version: str,
) -> None:
    s3_region = S3Path(repository_path).region
    exe_name = _parse_package_name(package_name)
    # TODO: Remove the hard coded path
    exe_local_path = DEFAULT_EXE_FOLDER + exe_name
    exe_s3_path = f"{repository_path}{package_name}/{version}/{exe_name}"
    logger.info(f"Downloading executables from {exe_s3_path}")
    storage_svc = S3StorageService(s3_region)
    storage_svc.copy(exe_s3_path, exe_local_path)


def _parse_package_name(package_name: str) -> str:
    # Some existing packages are like private_lift/lift, so we have to split it by slash
    return package_name.split("/")[-1]


def _read_config(
    config_name: str,
    argument: Optional[str],
    env_var: str,
    default_val: str,
):
    if argument:
        logger.info(f"Read {config_name} from program arguments...")
        return argument

    if os.getenv(env_var):
        logger.info(f"Read {config_name} from environment variables...")
        return os.getenv(env_var)

    logger.info(f"Read {config_name} from default value...")
    return default_val


def main():
    global logger
    s = schema.Schema(
        {
            "<package_name>": str,
            "--version": str,
            "--repository_path": schema.Or(None, schema.And(str, len)),
            "--exe_path": schema.Or(None, schema.And(str, len)),
            "--exe_args": schema.Or(None, schema.Use(str, len)),
            "--timeout": schema.Or(None, schema.Use(int)),
            "--log_path": schema.Or(None, schema.Use(Path)),
            "--verbose": bool,
            "--help": bool,
        }
    )

    arguments = s.validate(docopt(__doc__))

    logger = logging.getLogger(__name__)
    log_path = arguments["--log_path"]
    log_level = logging.DEBUG if arguments["--verbose"] else logging.INFO
    logging.basicConfig(filename=log_path, level=log_level)

    repository_path = _read_config(
        "repository_path",
        arguments["--repository_path"],
        ONEDOCKER_REPOSITORY_PATH,
        DEFAULT_REPOSITORY_PATH,
    )
    exe_path = _read_config(
        "exe_path",
        arguments["--exe_path"],
        ONEDOCKER_EXE_PATH,
        DEFAULT_EXE_FOLDER,
    )

    _run_package(
        repository_path=repository_path,
        exe_path=exe_path,
        package_name=arguments["<package_name>"],
        version=arguments["--version"],
        timeout=arguments["--timeout"],
        exe_args=arguments["--exe_args"],
    )


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import logging
from typing import Optional, Any, Dict

from fbpcp.entity.pce import PCE
from fbpcp.entity.pce_compute import PCECompute
from fbpcp.entity.pce_network import PCENetwork
from fbpcp.gateway.ec2 import EC2Gateway
from fbpcp.gateway.ecs import ECSGateway
from fbpcp.service.pce import PCEService


PCE_ID_KEY = "pce:pce-id"
SHARED_TASK_DEFINITION_PREFIX = "onedocker-task-shared-"


class AWSPCEService(PCEService):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        self.logger: logging.Logger = logging.getLogger(__name__)
        self.region = region
        self.access_key_id = access_key_id
        self.access_key_data = access_key_data
        self.config = config

    def get_pce(
        self,
        pce_id: str,
    ) -> PCE:
        pce_network = self._get_network(pce_id)
        pce_compute = self._get_compute(pce_id)
        return PCE(pce_id, self.region, pce_network, pce_compute)

    def _get_network(self, pce_id: str) -> PCENetwork:
        ec2_gateway = EC2Gateway(
            self.region, self.access_key_id, self.access_key_data, self.config
        )
        tags = {PCE_ID_KEY: pce_id}
        vpcs = ec2_gateway.describe_vpcs(tags=tags)
        vpc = vpcs[0] if vpcs else None
        subnets = ec2_gateway.describe_subnets(tags=tags)
        firewall_rulesets = ec2_gateway.describe_security_groups(tags=tags)
        route_tables = ec2_gateway.describe_route_tables(tags=tags)
        route_table = route_tables[0] if route_tables else None
        vpc_peering = None
        if vpc:
            vpc_peerings = ec2_gateway.describe_vpc_peerings(
                vpc_id=vpc.vpc_id, tags=tags
            )
            vpc_peering = vpc_peerings[0] if vpc_peerings else None
        return PCENetwork(
            region=self.region,
            vpc=vpc,
            subnets=subnets,
            route_table=route_table,
            vpc_peering=vpc_peering,
            firewall_rulesets=firewall_rulesets,
        )

    def _get_compute(self, pce_id: str) -> PCECompute:
        ecs_gateway = ECSGateway(
            self.region, self.access_key_id, self.access_key_data, self.config
        )
        tags = {PCE_ID_KEY: pce_id}
        clusters = ecs_gateway.describe_clusters(tags=tags)
        cluster = clusters[0] if clusters else None
        container_definitions = ecs_gateway.describe_task_definitions(tags=tags)
        if container_definitions:
            container_definition = container_definitions[0]
        else:
            container_definition = ecs_gateway.describe_task_definition(
                SHARED_TASK_DEFINITION_PREFIX + self.region
            )
        return PCECompute(self.region, cluster, container_definition)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass, field
from typing import Dict, List


@dataclass
class ContainerDefinition:
    id: str
    image: str
    cpu: int
    memory: int
    entry_point: List[str]
    environment: Dict[str, str]
    task_role_id: str
    tags: Dict[str, str] = field(default_factory=lambda: {})



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import os
from typing import Any, Dict, List, Optional

import boto3
from fbpcp.decorator.error_handler import error_handler
from fbpcp.gateway.aws import AWSGateway
from tqdm.auto import tqdm


class S3Gateway(AWSGateway):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(region, access_key_id, access_key_data, config)
        # pyre-ignore
        self.client = boto3.client("s3", region_name=self.region, **self.config)

    @error_handler
    def create_bucket(self, bucket: str, region: Optional[str] = None) -> None:
        region = region if region is not None else self.region
        self.client.create_bucket(
            Bucket=bucket, CreateBucketConfiguration={"LocationConstraint": region}
        )

    @error_handler
    def delete_bucket(self, bucket: str) -> None:
        self.client.delete_bucket(Bucket=bucket)

    @error_handler
    def upload_file(self, file_name: str, bucket: str, key: str) -> None:
        file_size = os.path.getsize(file_name)
        self.client.upload_file(
            file_name,
            bucket,
            key,
            Callback=self.ProgressPercentage(file_name, file_size),
        )

    @error_handler
    def download_file(self, bucket: str, key: str, file_name: str) -> None:
        file_size = self.get_object_size(bucket, key)
        self.client.download_file(
            bucket,
            key,
            file_name,
            Callback=self.ProgressPercentage(file_name, file_size),
        )

    @error_handler
    def put_object(self, bucket: str, key: str, data: str) -> None:
        self.client.put_object(Bucket=bucket, Key=key, Body=data.encode())

    @error_handler
    def get_object(self, bucket: str, key: str) -> str:
        res = self.client.get_object(Bucket=bucket, Key=key)
        return res["Body"].read().decode()

    @error_handler
    def get_object_size(self, bucket: str, key: str) -> int:
        return self.client.head_object(Bucket=bucket, Key=key)["ContentLength"]

    @error_handler
    def get_object_info(self, bucket: str, key: str) -> Dict[str, Any]:
        return self.client.get_object(Bucket=bucket, Key=key)

    @error_handler
    def list_object2(self, bucket: str, key: str) -> List[str]:
        paginator = self.client.get_paginator("list_objects_v2")
        pages = paginator.paginate(Bucket=bucket, Prefix=key)

        key_list = []
        for page in pages:
            for content in page["Contents"]:
                key_list.append(content["Key"])

        return key_list

    @error_handler
    def list_folders(self, bucket: str, key: str) -> List[str]:
        key = key + "/"
        response = self.client.list_objects_v2(Bucket=bucket, Prefix=key, Delimiter="/")
        return [
            prefix_dict["Prefix"][len(key) : -1]
            for prefix_dict in response.get("CommonPrefixes")
        ]

    @error_handler
    def delete_object(self, bucket: str, key: str) -> None:
        self.client.delete_object(Bucket=bucket, Key=key)

    @error_handler
    def object_exists(self, bucket: str, key: str) -> bool:
        try:
            # Result intentionally discarded
            _ = self.client.head_object(Bucket=bucket, Key=key)
            return True
        except Exception:
            return False

    @error_handler
    def copy(
        self, source_bucket: str, source_key: str, dest_bucket: str, dest_key: str
    ) -> None:
        source = {"Bucket": source_bucket, "Key": source_key}
        self.client.copy(source, dest_bucket, dest_key)

    class ProgressPercentage(object):
        def __init__(self, file_name: str, file_size: int) -> None:
            self._progressbar = tqdm(total=file_size, desc=file_name)

        def __call__(self, bytes_amount: int) -> None:
            self._progressbar.update(bytes_amount)

        def __del__(self) -> None:
            self._progressbar.close()



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Optional, Dict, List, Any

import boto3
from fbpcp.decorator.error_handler import error_handler
from fbpcp.entity.log_event import LogEvent
from fbpcp.gateway.aws import AWSGateway


class CloudWatchGateway(AWSGateway):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(region, access_key_id, access_key_data, config)
        # pyre-ignore
        self.client = boto3.client("logs", region_name=self.region, **self.config)

    @error_handler
    def get_log_events(
        self,
        log_group: str,
        log_stream: str,
        start_time: int = 0,
    ) -> List[LogEvent]:
        events = self.client.get_log_events(
            logGroupName=log_group,
            logStreamName=log_stream,
            startTime=start_time,
        )["events"]

        return [LogEvent(event["timestamp"], event["message"]) for event in events]



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import abc
from typing import Dict, List, Optional

from fbpcp.entity.container_instance import ContainerInstance
from fbpcp.error.pcp import PcpError


class ContainerService(abc.ABC):
    @abc.abstractmethod
    def get_region(
        self,
    ) -> str:
        pass

    @abc.abstractmethod
    def get_cluster(
        self,
    ) -> str:
        pass

    @abc.abstractmethod
    def create_instance(
        self,
        container_definition: str,
        cmd: str,
        env_vars: Optional[Dict[str, str]] = None,
    ) -> ContainerInstance:
        pass

    @abc.abstractmethod
    def create_instances(
        self,
        container_definition: str,
        cmds: List[str],
        env_vars: Optional[Dict[str, str]] = None,
    ) -> List[ContainerInstance]:
        pass

    @abc.abstractmethod
    def get_instance(self, instance_id: str) -> ContainerInstance:
        pass

    @abc.abstractmethod
    def get_instances(self, instance_ids: List[str]) -> List[ContainerInstance]:
        pass

    @abc.abstractmethod
    def cancel_instance(self, instance_id: str) -> None:
        pass

    @abc.abstractmethod
    def cancel_instances(self, instance_ids: List[str]) -> List[Optional[PcpError]]:
        pass



# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import logging
from typing import List, Optional, Dict, Any

import boto3


class EC2Gateway:
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        self.logger: logging.Logger = logging.getLogger(__name__)
        self.region = region
        self.config: Dict[str, Any] = config or {}

        if access_key_id is not None:
            self.config["aws_access_key_id"] = access_key_id

        if access_key_data is not None:
            self.config["aws_secret_access_key"] = access_key_data

        self.client = boto3.client("ec2", region_name=self.region, **self.config)

    def describe_availability_zones(
        self,
        tags: Optional[Dict[str, str]] = None,
    ) -> List[str]:
        response = self.client.describe_availability_zones()

        return [aws_az["ZoneName"] for aws_az in response["AvailabilityZones"]]



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import List, Optional, Dict, Any

import boto3
from fbpcp.decorator.error_handler import error_handler
from fbpcp.entity.firewall_ruleset import FirewallRuleset
from fbpcp.entity.route_table import RouteTable
from fbpcp.entity.subnet import Subnet
from fbpcp.entity.vpc_instance import Vpc
from fbpcp.entity.vpc_peering import VpcPeering
from fbpcp.gateway.aws import AWSGateway
from fbpcp.mapper.aws import (
    map_ec2vpc_to_vpcinstance,
    map_ec2subnet_to_subnet,
    map_ec2routetable_to_routetable,
    map_ec2securitygroup_to_firewallruleset,
    map_ec2vpcpeering_to_vpcpeering,
)
from fbpcp.util.aws import convert_vpc_tags_to_filter


class EC2Gateway(AWSGateway):
    def __init__(
        self,
        region: str,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(region, access_key_id, access_key_data, config)
        # pyre-ignore
        self.client = boto3.client("ec2", region_name=self.region, **self.config)

    @error_handler
    def describe_vpcs(
        self,
        vpc_ids: Optional[List[str]] = None,
        tags: Optional[Dict[str, str]] = None,
    ) -> List[Vpc]:
        if vpc_ids is None:
            vpc_ids = []
        filters = convert_vpc_tags_to_filter(tags)
        response = self.client.describe_vpcs(VpcIds=vpc_ids, Filters=filters)
        return [map_ec2vpc_to_vpcinstance(vpc) for vpc in response["Vpcs"]]

    @error_handler
    def describe_vpc(self, vpc_id: str) -> Vpc:
        return self.describe_vpcs([vpc_id])[0]

    @error_handler
    def list_vpcs(self) -> List[str]:
        all_vpcs = self.client.describe_vpcs()
        return [vpc["VpcId"] for vpc in all_vpcs["Vpcs"]]

    @error_handler
    def describe_subnets(
        self,
        vpc_id: Optional[str] = None,
        tags: Optional[Dict[str, str]] = None,
    ) -> List[Subnet]:
        filters = convert_vpc_tags_to_filter(tags, vpc_id)
        response = self.client.describe_subnets(Filters=filters)
        return [map_ec2subnet_to_subnet(subnet) for subnet in response["Subnets"]]

    @error_handler
    def describe_route_tables(
        self,
        vpc_id: Optional[str] = None,
        tags: Optional[Dict[str, str]] = None,
    ) -> List[RouteTable]:
        filters = convert_vpc_tags_to_filter(tags, vpc_id)
        response = self.client.describe_route_tables(Filters=filters)
        return [
            map_ec2routetable_to_routetable(route_table)
            for route_table in response["RouteTables"]
        ]

    @error_handler
    def describe_security_groups(
        self,
        vpc_id: Optional[str] = None,
        tags: Optional[Dict[str, str]] = None,
    ) -> List[FirewallRuleset]:
        filters = convert_vpc_tags_to_filter(tags, vpc_id)
        response = self.client.describe_security_groups(Filters=filters)
        return [
            map_ec2securitygroup_to_firewallruleset(security_group)
            for security_group in response["SecurityGroups"]
        ]

    @error_handler
    def describe_vpc_peerings(
        self,
        vpc_id: str,
        tags: Optional[Dict[str, str]] = None,
    ) -> List[VpcPeering]:
        filters = convert_vpc_tags_to_filter(tags)
        response = self.client.describe_vpc_peering_connections(Filters=filters)
        return [
            map_ec2vpcpeering_to_vpcpeering(vpc_peering, vpc_id)
            for vpc_peering in response["VpcPeeringConnections"]
        ]



# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

"""


Usage:
    pce_validator --region=<region> --key-id=<key_id> --key-data=<key_data> --pce-id=<pce_id>

Options:
    --region=<region>       (AWS) Region name
    --key-id=<key_id>       Key id
    --key-data=<key_data>   Key data
    --pce-id=<pce_id>       PCE id
"""


import logging

from docopt import docopt
from fbpcp.service.pce_aws import AWSPCEService
from pce.validator.validation_suite import ValidationSuite
from schema import Schema


def validate_pce(region: str, key_id: str, key_data: str, pce_id: str) -> None:
    pce_service = AWSPCEService(region, key_id, key_data, None)
    pce = pce_service.get_pce(pce_id)
    logging.info(f"PCE {pce}")
    validator = ValidationSuite(region, key_id, key_data, None)

    failed_results = validator.validate_network_and_compute(pce)
    if failed_results:
        logging.error(
            f"Validation failed for PCE {pce_id}:\n{ValidationSuite.summarize_errors(failed_results)}"
        )
    else:
        print("Your PCE environments are set up correctly.")


def main():
    logging.basicConfig(level=logging.INFO)

    s = Schema({"--region": str, "--key-id": str, "--key-data": str, "--pce-id": str})
    arguments = s.validate(docopt(__doc__))

    region = arguments["--region"]
    key_id = arguments["--key-id"]
    key_data = arguments["--key-data"]
    pce_id = arguments["--pce-id"]
    validate_pce(region, key_id, key_data, pce_id)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import logging
from typing import Any, Dict, List, Optional, Tuple

from fbpcp.entity.container_instance import ContainerInstance
from fbpcp.error.pcp import PcpError
from fbpcp.gateway.ecs import ECSGateway
from fbpcp.metrics.emitter import MetricsEmitter
from fbpcp.service.container import ContainerService


class AWSContainerService(ContainerService):
    def __init__(
        self,
        region: str,
        cluster: str,
        subnets: Optional[List[str]] = None,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
        metrics: Optional[MetricsEmitter] = None,
    ) -> None:
        self.logger: logging.Logger = logging.getLogger(__name__)
        self.region = region
        self.cluster = cluster
        self.subnets = subnets
        self.ecs_gateway = ECSGateway(
            region, access_key_id, access_key_data, config, metrics
        )

    def get_region(
        self,
    ) -> str:
        return self.region

    def get_cluster(
        self,
    ) -> str:
        return self.cluster

    def create_instance(
        self,
        container_definition: str,
        cmd: str,
        env_vars: Optional[Dict[str, str]] = None,
    ) -> ContainerInstance:
        task_definition, container = self._split_container_definition(
            container_definition
        )

        if not self.subnets:
            raise PcpError(
                "No subnets specified. It's required to create container instances."
            )

        return self.ecs_gateway.run_task(
            task_definition, container, cmd, self.cluster, self.subnets, env_vars
        )

    def create_instances(
        self,
        container_definition: str,
        cmds: List[str],
        env_vars: Optional[Dict[str, str]] = None,
    ) -> List[ContainerInstance]:
        instances = [
            self.create_instance(container_definition, cmd, env_vars) for cmd in cmds
        ]
        self.logger.info(
            f"AWSContainerService created {len(instances)} containers successfully"
        )
        return instances

    def get_instance(self, instance_id: str) -> ContainerInstance:
        return self.ecs_gateway.describe_task(self.cluster, instance_id)

    def get_instances(self, instance_ids: List[str]) -> List[ContainerInstance]:
        return self.ecs_gateway.describe_tasks(self.cluster, instance_ids)

    def cancel_instance(self, instance_id: str) -> None:
        return self.ecs_gateway.stop_task(cluster=self.cluster, task_id=instance_id)

    def cancel_instances(self, instance_ids: List[str]) -> List[Optional[PcpError]]:
        res = []
        for instance_id in instance_ids:
            try:
                res.append(self.cancel_instance(instance_id))
            except PcpError as err:
                res.append(err)

        return res

    def _split_container_definition(self, container_definition: str) -> Tuple[str, str]:
        """
        container_definition = task_definition#container
        """
        s = container_definition.split("#")
        return (s[0], s[1])



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import abc

from fbpcp.entity.mpc_instance import MPCInstance


class MPCInstanceRepository(abc.ABC):
    @abc.abstractmethod
    def create(self, instance: MPCInstance) -> None:
        pass

    @abc.abstractmethod
    def read(self, instance_id: str) -> MPCInstance:
        pass

    @abc.abstractmethod
    def update(self, instance: MPCInstance) -> None:
        pass

    @abc.abstractmethod
    def delete(self, instance_id: str) -> None:
        pass



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

"""Metrics Decorators

The decorators in this file are designed to deocrate classes that implement MetricsGetter.
"""

import asyncio
import functools
import time
from typing import Callable

from fbpcp.metrics.getter import MetricsGetter


def request_counter(metrics_name: str) -> Callable:
    def wrap(f: Callable):
        @functools.wraps(f)
        def wrapper_sync(self: MetricsGetter, *args, **kwargs):
            if self.has_metrics():
                self.get_metrics().count(metrics_name, 1)
            return f(self, *args, **kwargs)

        @functools.wraps(f)
        async def wrapper_async(self: MetricsGetter, *args, **kwargs):
            if self.has_metrics():
                self.get_metrics().count(metrics_name, 1)
            return await f(self, *args, **kwargs)

        return wrapper_async if asyncio.iscoroutinefunction(f) else wrapper_sync

    return wrap


def error_counter(metrics_name: str) -> Callable:
    def wrap(f: Callable):
        @functools.wraps(f)
        def wrapper_sync(self: MetricsGetter, *args, **kwargs):
            try:
                return f(self, *args, **kwargs)
            except Exception as err:
                if self.has_metrics():
                    self.get_metrics().count(metrics_name, 1)
                raise err

        @functools.wraps(f)
        async def wrapper_async(self: MetricsGetter, *args, **kwargs):
            try:
                return await f(self, *args, **kwargs)
            except Exception as err:
                if self.has_metrics():
                    self.get_metrics().count(metrics_name, 1)
                raise err

        return wrapper_async if asyncio.iscoroutinefunction(f) else wrapper_sync

    return wrap


def duration_time(metrics_name: str) -> Callable:
    def wrap(f: Callable):
        @functools.wraps(f)
        def wrapper_sync(self: MetricsGetter, *args, **kwargs):
            start = time.perf_counter_ns()
            res = f(self, *args, **kwargs)
            end = time.perf_counter_ns()

            if self.has_metrics():
                self.get_metrics().gauge(metrics_name, int((end - start) / 1e6))

            return res

        @functools.wraps(f)
        async def wrapper_async(self: MetricsGetter, *args, **kwargs):
            start = time.perf_counter_ns()
            res = await f(self, *args, **kwargs)
            end = time.perf_counter_ns()

            if self.has_metrics():
                self.get_metrics().gauge(metrics_name, int((end - start) / 1e6))

            return res

        return wrapper_async if asyncio.iscoroutinefunction(f) else wrapper_sync

    return wrap



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass
from enum import Enum
from typing import Optional

from dataclasses_json import dataclass_json


class ContainerInstanceStatus(Enum):
    UNKNOWN = "UNKNOWN"
    STARTED = "STARTED"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"


@dataclass_json
@dataclass
class ContainerInstance:
    instance_id: str
    ip_address: Optional[str] = None
    status: ContainerInstanceStatus = ContainerInstanceStatus.UNKNOWN



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

import os
import signal
import subprocess
from typing import Optional


def run_cmd(cmd: str, timeout: Optional[int]) -> int:
    # The handler dealing signal SIGINT, which could be Ctrl + C from user's terminal
    def _handler(signum, frame):
        raise InterruptedError

    signal.signal(signal.SIGINT, _handler)
    """
     If start_new_session is true the setsid() system call will be made in the
     child process prior to the execution of the subprocess, which makes sure
     every process in the same process group can be killed by OS if timeout occurs.
     note: setsid() will set the pgid to its pid.
    """
    with subprocess.Popen(cmd, shell=True, start_new_session=True) as proc:
        try:
            proc.communicate(timeout=timeout)
        except (subprocess.TimeoutExpired, InterruptedError) as e:
            proc.terminate()
            os.killpg(proc.pid, signal.SIGTERM)
            raise e

        return proc.wait()



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import abc

from fbpcp.metrics.emitter import MetricsEmitter


class MetricsGetter(abc.ABC):
    @abc.abstractmethod
    def has_metrics(self) -> bool:
        pass

    @abc.abstractmethod
    def get_metrics(self) -> MetricsEmitter:
        pass



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

from typing import Callable

from botocore.exceptions import ClientError
from fbpcp.error.mapper.aws import map_aws_error
from fbpcp.error.pcp import PcpError


def error_handler(f: Callable) -> Callable:
    def wrap(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except PcpError as err:
            raise err from None
        except ClientError as err:
            raise map_aws_error(err) from None
        except Exception as err:
            raise PcpError(err) from None

    return wrap



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.


"""
CLI for uploading an executable to one docker repo


Usage:
    onedocker-cli upload --config=<config> --package_name=<package_name> --package_dir=<package_dir> [--version=<version>] [options]
    onedocker-cli test --package_name=<package_name> --cmd_args=<cmd_args> --config=<config> [--timeout=<timeout> --version=<version>][options]
    onedocker-cli show --package_name=<package_name> --config=<config> [--version=<version>] [options]
    onedocker-cli stop --container=<container_id> --config=<config> [options]

Options:
    -h --help                Show this help
    --log_path=<path>        Override the default path where logs are saved
    --verbose                Set logging level to DEBUG
"""

import asyncio
import logging
import os
import time
from pathlib import Path, PurePath
from typing import Any, Dict, Optional

import schema
from docopt import docopt
from fbpcp.entity.container_instance import ContainerInstanceStatus
from fbpcp.service.container import ContainerService
from fbpcp.service.log import LogService
from fbpcp.service.onedocker import OneDockerService
from fbpcp.service.storage import StorageService
from fbpcp.util import reflect
from fbpcp.util import yaml
from onedocker.repository.onedocker_package import (
    OneDockerPackageRepository,
)


logger = None
onedocker_svc = None
container_svc = None
onedocker_package_repo = None
log_svc = None
task_definition = None
repository_path = None

DEFAULT_BINARY_VERSION = "latest"
DEFAULT_TIMEOUT = 18000


def _upload(
    package_dir: str,
    package_name: str,
    version: str,
) -> None:
    logger.info(
        f" Starting uploading package {package_name} at '{package_dir}', version {version}..."
    )
    onedocker_package_repo.upload(package_name, version, package_dir)
    logger.info(f" Finished uploading '{package_name}, version {version}'.\n")


def _test(
    package_name: str,
    version: str,
    cmd_args: str,
    timeout: int,
) -> None:
    logger.info(" Running test container ...")
    container = onedocker_svc.start_container(
        package_name=package_name,
        version=version,
        cmd_args=cmd_args,
        timeout=DEFAULT_TIMEOUT,
    )
    container = asyncio.run(
        onedocker_svc.wait_for_pending_container(container.instance_id)
    )
    logger.info(container)
    log_path = log_svc.get_log_path(container)
    start_time = 0
    while container.status == ContainerInstanceStatus.STARTED:
        container = container_svc.get_instance(container.instance_id)
        log_events = log_svc.fetch(log_path, start_time)

        for event in log_events:
            logger.info(event.message)

        if log_events:
            start_time = log_events[-1].timestamp + 1
        # temparorily set the process to be periodlly fetching and showing the log
        time.sleep(5)


def _show(
    package_name: str,
    version: Optional[str] = None,
) -> None:
    logger.info(
        f"Show package [{package_name}], version {version} information on storage "
    )

    if version:
        package_info = onedocker_package_repo.get_package_info(package_name, version)
        print(
            f" Package [{package_info.package_name}], version {package_info.version}: Last modified: {package_info.last_modified}; Size: {package_info.package_size} bytes"
        )
    else:
        package_versions = onedocker_package_repo.get_package_versions(package_name)
        print(
            f" All available versions for package {package_name} : {package_versions}"
        )
        for version in package_versions:
            package_info = onedocker_package_repo.get_package_info(
                package_name, version
            )
            print(
                f"Package [{package_info.package_name}], version {package_info.version}: Last modified: {package_info.last_modified}; Size: {package_info.package_size} bytes"
            )


def _stop(container_id: str) -> None:
    logger.info(f"Stopping container {container_id} ...")
    errors = onedocker_svc.stop_containers([container_id])
    if errors[0] is None:
        logger.info(f"Container {container_id} has been stopped")
    else:
        logger.info(
            f"Encountered errors when stopping container {container_id}: {errors[0]}"
        )


def _build_storage_service(config: Dict[str, Any]) -> StorageService:
    storage_class = reflect.get_class(config["class"])
    return storage_class(**config["constructor"])


def _build_container_service(config: Dict[str, Any]) -> ContainerService:
    container_class = reflect.get_class(config["class"])
    return container_class(**config["constructor"])


def _build_log_service(config: Dict[str, Any]) -> LogService:
    log_class = reflect.get_class(config["class"])
    return log_class(**config["constructor"])


def _build_exe_s3_path(repository_path: str, package_name: str, version: str) -> str:
    return f"{repository_path}{package_name}/{version}/{package_name.split('/')[-1]}"


def main():
    global container_svc, onedocker_svc, onedocker_package_repo, log_svc, logger, task_definition, repository_path
    s = schema.Schema(
        {
            "upload": bool,
            "test": bool,
            "show": bool,
            "stop": bool,
            "--config": schema.And(schema.Use(PurePath), os.path.exists),
            "--package_name": schema.Or(None, schema.And(str, len)),
            "--package_dir": schema.Or(None, schema.And(str, len)),
            "--cmd_args": schema.Or(None, schema.And(str, len)),
            "--container": schema.Or(None, schema.And(str, len)),
            "--log_path": schema.Or(None, schema.Use(Path)),
            "--verbose": bool,
            "--help": bool,
            "--version": schema.Or(None, schema.And(str, len)),
            "--timeout": schema.Or(None, schema.Use(int)),
        }
    )

    arguments = s.validate(docopt(__doc__))
    logger = logging.getLogger(__name__)
    log_path = arguments["--log_path"]
    log_level = logging.DEBUG if arguments["--verbose"] else logging.INFO
    logging.basicConfig(filename=log_path, level=log_level)

    package_name = arguments["--package_name"]
    package_dir = arguments["--package_dir"]
    version = (
        arguments["--version"] if arguments["--version"] else DEFAULT_BINARY_VERSION
    )

    config = yaml.load(Path(arguments["--config"])).get("onedocker-cli")
    task_definition = config["setting"]["task_definition"]
    repository_path = config["setting"]["repository_path"]

    storage_svc = _build_storage_service(config["dependency"]["StorageService"])
    container_svc = _build_container_service(config["dependency"]["ContainerService"])
    onedocker_svc = OneDockerService(container_svc, task_definition)
    onedocker_package_repo = OneDockerPackageRepository(storage_svc, repository_path)
    log_svc = _build_log_service(config["dependency"]["LogService"])
    if arguments["upload"]:
        _upload(package_dir, package_name, version)
    elif arguments["test"]:
        timeout = arguments["--timeout"] if arguments["--timeout"] else DEFAULT_TIMEOUT
        _test(package_name, version, arguments["--cmd_args"], timeout)
    elif arguments["show"]:
        _show(package_name, arguments["--version"])
    elif arguments["stop"]:
        _stop(arguments["--container"])


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from dataclasses import dataclass, field
from enum import Enum
from typing import Dict


class VpcPeeringState(Enum):
    PENDING_ACCEPTANCE = "PENDING_ACCEPTANCE"
    ACTIVE = "ACTIVE"
    REJECTED = "REJECTED"
    NOT_READY = "NOT_READY"


class VpcPeeringRole(Enum):
    REQUESTER = "REQUESTER"
    ACCEPTER = "ACCEPTER"


@dataclass
class VpcPeering:
    id: str
    status: VpcPeeringState
    role: VpcPeeringRole
    requester_vpc_id: str
    accepter_vpc_id: str
    tags: Dict[str, str] = field(default_factory=dict)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.


from setuptools import setup, find_packages

install_requires = [
    "boto3==1.11.11",
    "dataclasses-json==0.5.2",
    "pyyaml==5.4.1",
    "tqdm==4.55.1",
    "google-cloud-storage==1.30.0",
    "docopt==0.6.2",
    "schema==0.7.0",
    "psutil==5.8.0",
]

with open("README.md", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="fbpcp",
    version="0.2.0",
    description="Facebook Private Computation Platform",
    author="Facebook",
    author_email="researchtool-help@fb.com",
    url="https://github.com/facebookresearch/fbpcp",
    install_requires=install_requires,
    packages=find_packages(),
    long_description_content_type="text/markdown",
    long_description=long_description,
    python_requires=">=3.8",
)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass
from typing import List


@dataclass
class MPCGameArgument:
    name: str
    required: bool


@dataclass
class MPCGameConfig:
    game_name: str
    onedocker_package_name: str
    arguments: List[MPCGameArgument]



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import asyncio
import logging
from typing import Any, Dict, List, Optional

from fbpcp.entity.container_instance import ContainerInstance, ContainerInstanceStatus
from fbpcp.entity.mpc_instance import MPCInstance, MPCInstanceStatus, MPCParty
from fbpcp.repository.mpc_instance import MPCInstanceRepository
from fbpcp.service.container import ContainerService
from fbpcp.service.mpc_game import MPCGameService
from fbpcp.service.onedocker import OneDockerService
from fbpcp.util.typing import checked_cast

DEFAULT_BINARY_VERSION = "latest"


class MPCService:
    """MPCService is responsible for distributing a larger MPC game to multiple
    MPC workers
    """

    def __init__(
        self,
        container_svc: ContainerService,
        instance_repository: MPCInstanceRepository,
        task_definition: str,
        mpc_game_svc: MPCGameService,
    ) -> None:
        """Constructor of MPCService
        Keyword arguments:
        container_svc -- service to spawn container instances
        instance_repository -- repository to CRUD MPCInstance
        task_definition -- containers task definition
        mpc_game_svc -- service to generate package name and game arguments.
        """
        if container_svc is None or instance_repository is None or mpc_game_svc is None:
            raise ValueError(
                f"Dependency is missing. container_svc={container_svc}, mpc_game_svc={mpc_game_svc}, "
                f"instance_repository={instance_repository}"
            )

        self.container_svc = container_svc
        self.instance_repository = instance_repository
        self.task_definition = task_definition
        self.mpc_game_svc: MPCGameService = mpc_game_svc
        self.logger: logging.Logger = logging.getLogger(__name__)

        self.onedocker_svc = OneDockerService(self.container_svc, self.task_definition)

    """
    The game_args should be consistent with the game_config, which should be
    defined in caller's game repository.

    For example,
    If the game config looks like this:

    game_config = {
    "game": {
        "onedocker_package_name": "package_name",
        "arguments": [
            {"name": "input_filenames", "required": True},
            {"name": "input_directory", "required": True},
            {"name": "output_filenames", "required": True},
            {"name": "output_directory", "required": True},
            {"name": "concurrency", "required": True},
        ],
    },

    The game args should look like this:
    [
        # 1st container
        {
            "input_filenames": input_path_1,
            "input_directory": input_directory,
            "output_filenames": output_path_1,
            "output_directory": output_directory,
            "concurrency": cocurrency,
        },
        # 2nd container
        {
            "input_filenames": input_path_2,
            "input_directory": input_directory,
            "output_filenames": output_path_2,
            "output_directory": output_directory,
            "concurrency": cocurrency,
        },
    ]
    """

    def create_instance(
        self,
        instance_id: str,
        game_name: str,
        mpc_party: MPCParty,
        num_workers: int,
        server_ips: Optional[List[str]] = None,
        game_args: Optional[List[Dict[str, Any]]] = None,
    ) -> MPCInstance:
        self.logger.info(f"Creating MPC instance: {instance_id}")

        instance = MPCInstance(
            instance_id,
            game_name,
            mpc_party,
            num_workers,
            server_ips,
            [],
            MPCInstanceStatus.CREATED,
            game_args,
        )

        self.instance_repository.create(instance)
        return instance

    def start_instance(
        self,
        instance_id: str,
        output_files: Optional[List[str]] = None,
        server_ips: Optional[List[str]] = None,
        timeout: Optional[int] = None,
        version: str = DEFAULT_BINARY_VERSION,
    ) -> MPCInstance:
        return asyncio.run(
            self.start_instance_async(
                instance_id, output_files, server_ips, timeout, version
            )
        )

    async def start_instance_async(
        self,
        instance_id: str,
        output_files: Optional[List[str]] = None,
        server_ips: Optional[List[str]] = None,
        timeout: Optional[int] = None,
        version: str = DEFAULT_BINARY_VERSION,
    ) -> MPCInstance:
        """To run a distributed MPC game
        Keyword arguments:
        instance_id -- unique id to identify the MPC instance
        """
        instance = self.instance_repository.read(instance_id)
        self.logger.info(f"Starting MPC instance: {instance_id}")

        if instance.mpc_party is MPCParty.CLIENT and not server_ips:
            raise ValueError("Missing server_ips")

        # spin up containers
        self.logger.info("Spinning up container instances")
        game_args = instance.game_args
        instance.containers = await self._spin_up_containers_onedocker(
            instance.game_name,
            instance.mpc_party,
            instance.num_workers,
            game_args,
            server_ips,
            timeout,
            version,
        )

        if len(instance.containers) != instance.num_workers:
            self.logger.warning(
                f"Instance {instance_id} has {len(instance.containers)} containers spun up, but expecting {instance.num_workers} containers!"
            )

        if instance.mpc_party is MPCParty.SERVER:
            ip_addresses = [
                checked_cast(str, instance.ip_address)
                for instance in instance.containers
            ]
            instance.server_ips = ip_addresses

        instance.status = MPCInstanceStatus.STARTED
        self.instance_repository.update(instance)

        return instance

    def stop_instance(self, instance_id: str) -> MPCInstance:
        instance = self.instance_repository.read(instance_id)
        container_ids = [instance.instance_id for instance in instance.containers]
        if container_ids:
            errors = self.onedocker_svc.stop_containers(container_ids)
            error_msg = list(filter(lambda _: _[1], zip(container_ids, errors)))

            if error_msg:
                self.logger.error(
                    f"We encountered errors when stopping containers: {error_msg}"
                )

        instance.status = MPCInstanceStatus.CANCELED
        self.instance_repository.update(instance)
        self.logger.info(f"MPC instance {instance_id} has been successfully canceled.")

        return instance

    def get_instance(self, instance_id: str) -> MPCInstance:
        self.logger.info(f"Getting MPC instance: {instance_id}")
        return self.instance_repository.read(instance_id)

    def update_instance(self, instance_id: str) -> MPCInstance:
        instance = self.instance_repository.read(instance_id)

        self.logger.info(f"Updating MPC instance: {instance_id}")

        if instance.status in [
            MPCInstanceStatus.COMPLETED,
            MPCInstanceStatus.FAILED,
            MPCInstanceStatus.CANCELED,
        ]:
            return instance

        # skip if no containers registered under instance yet
        if instance.containers:
            instance.containers = self._update_container_instances(instance.containers)

            if len(instance.containers) != instance.num_workers:
                self.logger.warning(
                    f"Instance {instance_id} has {len(instance.containers)} containers after update, but expecting {instance.num_workers} containers!"
                )

            instance.status = self._get_instance_status(instance)
            self.instance_repository.update(instance)

        return instance

    async def _spin_up_containers_onedocker(
        self,
        game_name: str,
        mpc_party: MPCParty,
        num_containers: int,
        game_args: Optional[List[Dict[str, Any]]] = None,
        ip_addresses: Optional[List[str]] = None,
        timeout: Optional[int] = None,
        version: str = DEFAULT_BINARY_VERSION,
    ) -> List[ContainerInstance]:
        if game_args is not None and len(game_args) != num_containers:
            raise ValueError(
                "The number of containers is not consistent with the number of game argument dictionary."
            )
        if ip_addresses is not None and len(ip_addresses) != num_containers:
            raise ValueError(
                "The number of containers is not consistent with number of ip addresses."
            )
        cmd_tuple_list = []
        for i in range(num_containers):
            game_arg = game_args[i] if game_args is not None else {}
            server_ip = ip_addresses[i] if ip_addresses is not None else None
            cmd_tuple_list.append(
                self.mpc_game_svc.build_onedocker_args(
                    game_name=game_name,
                    mpc_party=mpc_party,
                    server_ip=server_ip,
                    **game_arg,
                )
            )
        cmd_args_list = [cmd_args for (package_name, cmd_args) in cmd_tuple_list]

        return await self.onedocker_svc.start_containers_async(
            task_definition=self.task_definition,
            package_name=cmd_tuple_list[0][0],
            version=version,
            cmd_args_list=cmd_args_list,
            timeout=timeout,
        )

    def _update_container_instances(
        self, containers: List[ContainerInstance]
    ) -> List[ContainerInstance]:
        ids = [container.instance_id for container in containers]
        return self.container_svc.get_instances(ids)

    def _get_instance_status(self, instance: MPCInstance) -> MPCInstanceStatus:
        if instance.status is MPCInstanceStatus.CANCELED:
            return instance.status
        status = MPCInstanceStatus.COMPLETED

        for container in instance.containers:
            if container.status == ContainerInstanceStatus.FAILED:
                return MPCInstanceStatus.FAILED
            if container.status == ContainerInstanceStatus.UNKNOWN:
                return MPCInstanceStatus.UNKNOWN
            if container.status == ContainerInstanceStatus.STARTED:
                status = MPCInstanceStatus.STARTED

        return status



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from dataclasses import dataclass, field
from typing import Dict, List


@dataclass
class FirewallRule:
    from_port: int
    to_port: int
    ip_protocol: str
    cidr: str


@dataclass
class FirewallRuleset:
    id: str
    vpc_id: str
    ingress: List[FirewallRule]
    egress: List[FirewallRule]
    tags: Dict[str, str] = field(default_factory=dict)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Dict, Any, List, Optional

from fbpcp.entity.container_instance import ContainerInstance
from fbpcp.entity.log_event import LogEvent
from fbpcp.gateway.cloudwatch import CloudWatchGateway
from fbpcp.service.log import LogService


class CloudWatchLogService(LogService):
    def __init__(
        self,
        log_group: str,
        region: str = "us-west-1",
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        self.cloudwatch_gateway = CloudWatchGateway(
            region, access_key_id, access_key_data, config
        )
        self.log_group = log_group

    def fetch(self, log_path: str, start_time: int = 0) -> List[LogEvent]:
        """Fetch logs"""
        return self.cloudwatch_gateway.get_log_events(
            self.log_group, log_path, start_time
        )

    def get_log_path(self, container_instance: ContainerInstance) -> str:
        return self.log_group[1:] + "/" + container_instance.instance_id.split("/")[-1]



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass, field
from enum import Enum
from typing import Dict

from dataclasses_json import dataclass_json


class ClusterStatus(Enum):
    ACTIVE = "ACTIVE"
    INACTIVE = "INACTIVE"
    UNKNOWN = "UNKNOWN"


@dataclass_json
@dataclass
class Cluster:
    cluster_arn: str
    cluster_name: str
    pending_tasks: int
    running_tasks: int
    status: ClusterStatus = ClusterStatus.UNKNOWN
    tags: Dict[str, str] = field(default_factory=dict)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from typing import Optional, Dict, Any

import boto3
from fbpcp.decorator.error_handler import error_handler
from fbpcp.entity.cloud_cost import CloudCost
from fbpcp.gateway.aws import AWSGateway
from fbpcp.mapper.aws import map_cecost_to_cloud_cost

COST_GRANULARITY = "DAILY"


class CostExplorerGateway(AWSGateway):
    def __init__(
        self,
        access_key_id: Optional[str] = None,
        access_key_data: Optional[str] = None,
        config: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(
            access_key_id=access_key_id, access_key_data=access_key_data, config=config
        )
        # pyre-ignore
        self.client = boto3.client("ce", **self.config)

    @error_handler
    def get_cost(
        self,
        start_date: str,
        end_date: str,
        region: Optional[str] = None,
    ) -> CloudCost:
        """
        Get cost between start_date and end_date from CostExplorer API using get_cost_and_usage()
        get_cost_and_usage() referece: https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ce.html#CostExplorer.Client.get_cost_and_usage
        :param start_date: start date for cost, required format "yyyy-mm-dd" (e.g "2020-12-01")
        :param end_date: end date for cost, required format "yyyy-mm-dd" (e.g "2020-12-01")
        :param region: region name as additional filter for cost.
        :return: CloudCost object that has the total cost and a list of CloudCostItem objects group by region and service. Unit of cost_amount is USD
        """

        page_token = None
        results_by_time = []
        while True:
            kwargs = {"NextPageToken": page_token} if page_token else {}
            kwargs.update(
                {
                    "Filter": {
                        "Dimensions": {
                            "Key": "REGION",
                            "Values": [region],
                        }
                    }
                }
                if region
                else {}
            )
            client_response = self.client.get_cost_and_usage(
                TimePeriod={"Start": start_date, "End": end_date},
                Granularity=COST_GRANULARITY,
                Metrics=["UnblendedCost"],
                GroupBy=[
                    {"Type": "DIMENSION", "Key": "SERVICE"},
                ],
                **kwargs,
            )
            results_by_time.extend(client_response.get("ResultsByTime"))
            page_token = client_response.get("NextPageToken")
            if not page_token:
                break

        return map_cecost_to_cloud_cost(results_by_time)



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import logging
from typing import Any, Dict, Optional, Tuple

from fbpcp.entity.mpc_game_config import MPCGameConfig
from fbpcp.entity.mpc_instance import MPCParty
from fbpcp.repository.mpc_game_repository import MPCGameRepository
from fbpcp.util.arg_builder import build_cmd_args

LIFT_GAME_NAME = "lift"
LIFT_AGGREGATOR_GAME_NAME = "aggregator"


class MPCGameService:
    def __init__(self, mpc_game_repository: MPCGameRepository) -> None:
        self.logger: logging.Logger = logging.getLogger(__name__)
        self.mpc_game_repository: MPCGameRepository = mpc_game_repository

    # returns package_name and cmd which includes only arguments (no executable)
    def build_onedocker_args(
        self,
        game_name: str,
        mpc_party: MPCParty,
        server_ip: Optional[str] = None,
        port: Optional[int] = None,
        **kwargs: object,
    ) -> Tuple[str, str]:
        mpc_game_config = self.mpc_game_repository.get_game(game_name)
        return (
            mpc_game_config.onedocker_package_name,
            self._build_cmd(
                mpc_game_config=mpc_game_config,
                mpc_party=mpc_party,
                server_ip=server_ip,
                port=port,
                **kwargs,
            ),
        )

    # returns cmd which includes only arguments (no executable)
    def _build_cmd(
        self,
        mpc_game_config: MPCGameConfig,
        mpc_party: MPCParty,
        server_ip: Optional[str] = None,
        port: Optional[int] = None,
        **kwargs: object,
    ) -> str:
        args = self._prepare_args(
            mpc_game_config=mpc_game_config,
            mpc_party=mpc_party,
            server_ip=server_ip,
            port=port,
            **kwargs,
        )
        return build_cmd_args(**args)

    def _prepare_args(
        self,
        mpc_game_config: MPCGameConfig,
        mpc_party: MPCParty,
        server_ip: Optional[str] = None,
        port: Optional[int] = None,
        **kwargs: object,
    ) -> Dict[str, Any]:
        all_arguments: Dict[str, Any] = {}

        # push MPC required arguments to dict all_arguments
        all_arguments["party"] = 1 if mpc_party == MPCParty.SERVER else 2

        if mpc_party == MPCParty.CLIENT:
            if server_ip is None:
                raise ValueError("Client must provide a server ip address.")
            all_arguments["server_ip"] = server_ip
        if port is not None:
            all_arguments["port"] = port

        # push game specific arguments to dict all_arguments
        for argument in mpc_game_config.arguments:
            key = argument.name
            value = kwargs.get(key)
            if value is None and argument.required:
                # Have to make game_name a special case for PL-Worker
                if key == "game_name":
                    all_arguments[key] = mpc_game_config.game_name
                else:
                    raise ValueError(f"Missing required argument {key}!")
            if value is not None:
                all_arguments[key] = value

        return all_arguments



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict


from shlex import quote


def build_cmd_args(
    **kwargs: object,
) -> str:
    return " ".join(
        [
            f"--{key}={quote(str(value))}"
            for key, value in kwargs.items()
            if value is not None
        ]
    )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict
from typing import List

from fbpcp.service.storage import StorageService
from onedocker.entity.package_info import PackageInfo


class OneDockerPackageRepository:
    def __init__(self, storage_svc: StorageService, repository_path: str) -> None:
        self.storage_svc = storage_svc
        self.repository_path = repository_path

    def _build_package_path(self, package_name: str, version: str) -> str:
        return f"{self.repository_path}{package_name}/{version}/{package_name.split('/')[-1]}"

    def upload(self, package_name: str, version: str, source: str) -> None:
        package_path = self._build_package_path(package_name, version)
        self.storage_svc.copy(source, package_path)

    def download(self, package_name: str, version: str, destination: str) -> None:
        package_path = self._build_package_path(package_name, version)
        self.storage_svc.copy(package_path, destination)

    def get_package_versions(
        self,
        package_name: str,
    ) -> List[str]:
        package_parent_path = f"{self.repository_path}{package_name}/"
        return self.storage_svc.list_folders(package_parent_path)

    def get_package_info(self, package_name: str, version: str) -> PackageInfo:
        package_path = self._build_package_path(package_name, version)

        if not self.storage_svc.file_exists(package_path):
            raise ValueError(
                f"Package {package_name}, version {version} not found in repository"
            )

        file_info = self.storage_svc.get_file_info(package_path)
        return PackageInfo(
            package_name=package_name,
            version=version,
            last_modified=file_info.last_modified,
            package_size=file_info.file_size,
        )



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

import abc

from fbpcp.entity.pce import PCE


class PCEService(abc.ABC):
    @abc.abstractmethod
    def get_pce(
        self,
        pce_id: str,
    ) -> PCE:
        pass



#!/usr/bin/env python3
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

# pyre-strict

from dataclasses import dataclass
from enum import Enum
from typing import Any, Dict, List, Optional

from fbpcp.entity.container_instance import ContainerInstance


class MPCParty(Enum):
    SERVER = "SERVER"
    CLIENT = "CLIENT"


class MPCInstanceStatus(Enum):
    UNKNOWN = "UNKNOWN"
    CREATED = "CREATED"
    STARTED = "STARTED"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"
    CANCELED = "CANCELED"


@dataclass
class MPCInstance:
    instance_id: str
    game_name: str
    mpc_party: MPCParty
    num_workers: int
    server_ips: Optional[List[str]]
    containers: List[ContainerInstance]
    status: MPCInstanceStatus
    game_args: Optional[List[Dict[str, Any]]]

    def get_instance_id(self) -> str:
        return self.instance_id
